/*
 * AliveLib.cpp
 *
 *  Created on: 2010/11/29
 *      Author: yotubata
 */
#include <unistd.h>
#include <iostream>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include "RtcControl.h"
#include "Tools.h"
#include "AliveLib.h"

#define DEF_BUFF_SIZE	512

#if 0
typedef struct _tagComponentInfo{
	std::string	namingServer;
	std::string	fileName;
	bool		aliveTarget;
} RelativeCmponentInfo;

typedef struct _tagPortInfo{
	std::string	componentName;
	std::string	portName;
} RelativePortInfo;
#else

struct RelativeCmponentInfo{
	RelativeCmponentInfo(){ namingServer = ""; fileName = ""; directoryName = ""; aliveTarget = false; }

	std::string	namingServer;
	std::string	fileName;
	std::string	directoryName;
	bool		aliveTarget;
};

struct RelativePortInfo{
	RelativePortInfo(){ componentName = ""; portName = ""; }

	std::string	componentName;
	std::string	portName;
};

#endif

typedef std::map< std::string, RelativeCmponentInfo > relativeCompList;
typedef relativeCompList::value_type relativeCompValType;
typedef relativeCompList::iterator relativeCompIterator;

#if 0

typedef std::vector< RelativePortInfo >	relativePortInfoList;
typedef std::map< std::string, relativePortInfoList > relativePortList;

#else

typedef std::map< std::string, RelativePortInfo > relativePortList;

#endif

typedef relativePortList::value_type relativePortValType;
typedef relativePortList::iterator relativePortIterator;

/*
 * グローバル変数定義
 */
RTC::RTObject_impl				*pBaseComp;
RTC::Manager					*pCompManager;
time_t							aliveOldData	= 0;
time_t							lastChangeTime	= 0;
RTC::TimedLong					aliveFlagData;
RTC::InPort<RTC::TimedLong>		aliveInPort("aliveIn", aliveFlagData);
RTC::OutPort<RTC::TimedLong>	aliveOutPort("aliveOut", aliveFlagData);
relativeCompList				relativeComponents;
relativePortList				relativePorts;

// ローカル関数宣言
void parseProperty(std::string keyName, std::string keyValue);
void treeDump(const coil::Properties* treeLeaf);
bool runComponent(std::string directoryName, std::string compFileName);
void wakeupComponent();
void connectComponent();

/**
 * @brief コンポーネント生存確認処理ライブラリの初期化を行う
 * @param pComp : 生存主張を行うコンポーネントのポインタ
 * @note  本関数は、生存確認用データポート（入力・出力の両方）を生成後、コンポーネントのプロパティから必要コンポーネント（接続先コンポーネント）を取得する。
 */
RTC::ReturnCode_t aliveLibInit(RTC::Manager *mgr, RTC::RTObject_impl *pComp)
{
	RTC::ReturnCode_t	result = RTC::RTC_OK;
	SDOPackage::NVList	nv;
	coil::Properties	compProps;

	if(pComp != 0){
		// コンポーネントポインタ退避
		pBaseComp		= pComp;
		pCompManager	= mgr;

		// ORBの取得
		corbaORB = pCompManager->getORB();

		// ネーミングサーバに接続
		connectNamingServer("localhost");

		// コンポーネントプロパティを取得
		compProps = pComp->getProperties();

		/*
		 * プロパティの分解を行う
		 */
		{
			std::vector<std::string>	propertys = compProps.propertyNames();
			std::string					keyVal, targetCompName;
			RelativePortInfo			portInfo;

			for(std::vector<std::string>::iterator it = propertys.begin(); it != propertys.end(); it++){
				// 関連のあるキーか確認する
				if(it->find("aliveLib.") == 0){
					// キー値取得
					keyVal = compProps[*it];
					// キー値のダンプ
					std::cout << keyVal << std::endl;
					// 関連のあるキーだった
					parseProperty(*it, keyVal);
				}
			}
			targetCompName = "";
			for(relativeCompIterator it = relativeComponents.begin(); it != relativeComponents.end(); it++){
				if(it->second.aliveTarget){
					targetCompName = it->first;
					break;
				}
			}
			// 監視対象コンポーネント・ポート情報を生成する
			if(targetCompName.length() > 0){
				portInfo.componentName	= targetCompName;
				portInfo.portName		= targetCompName + ".aliveOut";
				targetCompName = pBaseComp->getInstanceName();
				targetCompName = targetCompName + ".aliveIn";
				relativePorts[targetCompName] = portInfo;
			}
		}
#if 0
		// プロパティのダンプを行う
		{
			// 普通のダンプ
			std::cout << "--------- Normal Dump ----------" << std::endl;
			NVUtil::copyFromProperties(nv, compProps);
			NVUtil::dump(nv);
			std::cout << "--------------------------------" << std::endl;

			// ツリー構造ダンプ
			std::cout << "--------- Tree Dump ----------" << std::endl;
			treeDump(compProps.getRoot());
			std::cout << "------------------------------" << std::endl;
		}
#endif
		// 監視対象が設定されている場合は、ポートの生成を行いコールバックを設定する
		{
			// alive信号送信ポート
			pComp->addOutPort("aliveOut", aliveOutPort);
			// alive信号受信ポート
			pComp->addInPort("aliveIn", aliveInPort);
		}

		/*
		 * alive信号送受信用スレッドを生成するか？
		 * 現時点では、本体のonExecuteで呼び出してもらう事にする
		 */
		// ターゲットの取得
		wakeupComponent();
	}
	else{
		result = RTC::BAD_PARAMETER;
	}

	return result;
}

void parseProperty(std::string keyName, std::string keyValue)
{
	std::vector< std::string >	keyList, valList;
	RelativePortInfo			portInfo;

	/*
	 * プロパティサンプル
	 *
	 * aliveLib.need.RtComponentName : RtComponentBinaryName
	 * aliveLib.connect.DataInPortName : NamingName:Port.RtComponentName.port_name
	 * aliveLib.alive : RtComponentName
	 *
	 */
	keyList = string_token(keyName, " .");
	/*for(std::vector< std::string >::iterator it = keyList.begin(); it != keyList.end(); it++)*/{
		// キーの内容を確認して、処理の振り分けを行う
		if(keyList.size() > 2){
			// キー階層数正常
			if(keyList[1].compare("need") == 0){
				// 必要コンポーネント定義
				// パラメタ確認
				valList = string_token(keyValue, " ");

				// ディレクトリ移動の有無を確認する
				if(valList[0].compare("-d") == 0){
					// ディレクトリ移動を行う
					relativeComponents[keyList[2]].directoryName = valList[1];
					keyValue = "";
					for(unsigned int keyCnt = 2; keyCnt < valList.size(); keyCnt++){
						keyValue = keyValue + valList[keyCnt] + " ";
					}
				}
				/*else*/{
					// ディレクトリ指定なし
					relativeComponents[keyList[2]].fileName = keyValue;
					std::cout << keyValue << std::endl;
				}
			}
			if(keyList[1].compare("connect") == 0){
				// ポート接続情報定義
				valList = string_token(keyValue, " .");
				if(valList.size() > 2){
					relativeComponents[valList[1]].namingServer = valList[0];
					portInfo.componentName	= valList[1];
					portInfo.portName		= valList[2];
					relativePorts[keyList[2]] = portInfo;
				}
			}
		}
		if(keyList.size() > 1){
			if(keyList[1].compare("alive") == 0){
				// 監視対象コンポーネント定義
				relativeComponents[keyValue].aliveTarget = true;
			}
		}
	}

	return;
}

void connectComponent()
{
	for(relativePortIterator it = relativePorts.begin(); it != relativePorts.end(); it++){
		// 接続先情報の取得
		std::cout << "Connect Info" << std::endl;
		std::cout << pBaseComp->getInstanceName() << std::endl;
		std::cout << it->first << std::endl;
		std::cout << it->second.componentName << std::endl;
		std::cout << it->second.portName << std::endl;
		std::cout << "Connect Info End" << std::endl;
		linkComponent(pBaseComp->getInstanceName(), it->first, it->second.componentName, it->second.portName);
	}
}

void wakeupComponent()
{
	for(relativeCompIterator it = relativeComponents.begin(); it != relativeComponents.end(); it++){
		if(findComponent(it->first) == false){
			std::cout << "Find Component [ " << it->first << " ]" << std::endl;
			// コンポーネント未起動なので起動する・・・ただしlocalhostのやつだけ
			if(it->second.fileName.length() > 0)
				runComponent(it->second.directoryName, it->second.fileName);
		}
	}
}

bool runComponent(std::string directoryName, std::string compFileName)
{
	pid_t	cpid;
#ifdef USE_PIPE
	int		pipe_fd[2];
#endif
	std::string	compFile, cpidStr;

#ifdef USE_PIPE
	if (pipe(pipe_fd) == -1) {
		// パイプ作成エラー
		std::cout << "pipe error!" << std::endl;
		return false;
	}
#endif
	/*
	 * コンポーネントが登録されていないので起動する
	 * 起動には・・・自分を子プロセスで作動させて、プロセスイメージを変える
	 * 子プロセスのプロセスIDとコンポーネント名を退避させておく
	 */
	cpid = fork();
	if(cpid == -1){
		std::cout << "自分の起動に失敗した" << std::endl;
	}
	cpidStr = "" + cpid;
	if(cpid == 0){
		int		sys_ret;
#ifdef USE_PIPE
		int		cnt;
		ssize_t	readSize = 1, totalSize = 0;
		size_t	commandLineLen = 0;
		char	*lpszCommandLine = 0;
#endif
		std::cout << "CPID = " << getpid() << std::endl;
		std::cout << "Directory = " << directoryName << std::endl;
		std::cout << "Command = " << compFileName << std::endl;

		// 子プロセスが作動した
		// パイプ読み込み待機
		// パイプから取得した文字列でプロセスを起動する
#ifdef USE_PIPE
		close(pipe_fd[1]);
		cnt = 0;
		while(readSize > 0){
			commandLineLen = DEF_BUFF_SIZE * (cnt + 1);
			lpszCommandLine = (char*)realloc(lpszCommandLine, commandLineLen);
			readSize = read(pipe_fd[0], &(lpszCommandLine[totalSize]), commandLineLen);
			totalSize = totalSize + readSize;
			cnt++;
		}
		close(pipe_fd[0]);
		std::cout << "CommandLine = " << lpszCommandLine << std::endl;
#endif
		// コマンドライン文字列をパースして、コマンド名とパラメタに分解する
		{
			std::string	binaryName;
#ifdef USE_PIPE
			std::string	strCommandLine(lpszCommandLine);

			sys_ret = system(strCommandLine.c_str());
			free(lpszCommandLine);
#else
			sleep(5);
			std::cout << "System [ " << compFileName << " ]" << std::endl;
			sys_ret = chdir(directoryName.c_str());
			if(sys_ret == -1){
				printf( "エラー(%d):%s\n", errno, strerror(errno));
			}
#if 0
			sys_ret = system(compFileName.c_str());
#else
			binaryName = (string_token(compFileName, " "))[0];
			sys_ret = execl(binaryName.c_str(), compFileName.c_str(), NULL);
			std::cout << "exec fail" << std::endl;
			printf( "エラー(%d):%s\n\t%s\n", errno, strerror(errno), binaryName.c_str());
#endif
/*
			switch(errno){
			case E2BIG:
				std::cout << "環境変数 (envp) と引き数リスト (argv) の合計バイト数が大き過ぎる。" << std::endl;
				break;
			case EACCES:
				std::cout << "filename やスクリプトインタプリタ名の構成要素に検索許可 (search permission) が与えられていない (path_resolution(7) も参照すること)。" << std::endl;
				std::cout << "ファイルもしくはスクリプトのインタプリタが通常ファイル (regular file) でない。" << std::endl;
				std::cout << "ファイルやスクリプトや ELF インタプリタに 実行許可 (execute permission) が与えられていない。" << std::endl;
				std::cout << "ファイル・システムが noexec でマウントされている。" << std::endl;
				break;
			case EFAULT:
				std::cout << "filename がアクセス可能なアドレス空間の外を指している。" << std::endl;
				break;
			case EINVAL:
				std::cout << "ELF 実行形式で複数の PT_INTERP セグメントが存在する。 (すなわち複数のインタプリタを指定した。)" << std::endl;
				break;
			case EIO:
				std::cout << "I/O エラーが発生した。" << std::endl;
				break;
			case EISDIR:
				std::cout << "ELF インタプリタがディレクトリだった。" << std::endl;
				break;
			case ELIBBAD:
				std::cout << "ELF インタプリタが理解できるフォーマットでなかった。" << std::endl;
				break;
			case ELOOP:
				std::cout << "filename やスクリプトや ELF のインタプリタを解決する際に遭遇した シンボリック・リンクが多過ぎる。" << std::endl;
				break;
			case EMFILE:
				std::cout << "そのプロセスがオープンできるファイル数の上限まで既にオープンしている。" << std::endl;
				break;
			case ENAMETOOLONG:
				std::cout << "filename が長過ぎる。" << std::endl;
				break;
			case ENFILE:
				std::cout << "そのシステムでオープンできるファイル数の制限に達した。" << std::endl;
				break;
			case ENOENT:
				std::cout << "ファイル filename かスクリプトや ELF のインタプリタが存在しない。" << std::endl;
				break;
			case ENOEXEC:
				std::cout << "実行ファイルが理解できない形式であるか、違うアーキテクチャのものか、 その他のフォーマット・エラーにより実行ができなかった。" << std::endl;
				break;
			case ENOMEM:
				std::cout << "カーネルに十分なメモリがない。" << std::endl;
				break;
			case ENOTDIR:
				std::cout << "filename やスクリプトや ELF のインタプリタの構成要素がディレクトリでない。" << std::endl;
				break;
			case EPERM:
				std::cout << "ファイル・システムが nosuid でマウントされ、ユーザがスーパーユーザでなく、 ファイルに set-user-ID あるいは set-group-ID ビットが設定されている。" << std::endl;
				std::cout << "プロセスがトレースされ、ユーザがスーパーユーザでなく、 ファイルに set-user-ID あるいは set-group-ID ビットが設定されている。" << std::endl;
				break;
			case ETXTBSY:
				std::cout << "実行ファイルを書き込み用にオープンしているプロセスがある。" << std::endl;
				break;
			}
*/
#endif
			exit(EXIT_SUCCESS);
		}
	}
#ifdef USE_PIPE
	else{
		size_t		writeLen;
		std::string	commandLine;
		char	*lpszCommandLine = 0;

		// 継続処理・・・パイプに起動するコンポーネントとパラメタを流す
		std::cout << "WriteCommand = " << compFileName << std::endl;
		lpszCommandLine = (char*)realloc(lpszCommandLine, compFileName.length() + 2);
		memset(lpszCommandLine, 0, compFileName.length() + 2);
		strcpy(lpszCommandLine,compFileName.c_str());

		close(pipe_fd[0]);
		writeLen = write(pipe_fd[1], lpszCommandLine, compFileName.length() + 2);

		close(pipe_fd[1]);
	}
#else
	/*
	else{
		kill(cpid, SIGKILL);
	}
	*/
#endif

	return true;
}

void treeDump(const coil::Properties* treeLeaf)
{
	std::vector<coil::Properties*>	leafs;

	if(treeLeaf != 0){
		std::cout << "Name = " << treeLeaf->getName() << std::endl;
		std::cout << "Value = " << treeLeaf->getValue() << std::endl;
		leafs = treeLeaf->getLeaf();
		for(std::vector<coil::Properties*>::iterator it = leafs.begin(); it != leafs.end(); it++){
			treeDump(*it);
		}
	}
}

RTC::ReturnCode_t aliveLibActivated()
{
	RTC::ReturnCode_t	result = RTC::RTC_OK;

	// ポートの接続を行う
	{
#if 0
		/*
		 * よくわからないのだが・・・ネーミングサーバ名はとれなくて、インスタンス名？のようなものがとれた
		 */
		{
			std::vector< std::string >	namingServers = pCompManager->getNamingNames();
			std::cout << getInstanceName() << std::endl;
			std::cout << "------ Naming Servers ------" << std::endl;
			for(std::vector< std::string >::iterator it = namingServers.begin(); it != namingServers.end(); it++){
				std::cout << *it << std::endl;
			}
			std::cout << "----------------------------" << std::endl;
		}
#endif
		// ポートの接続
		connectComponent();
	}

	return result;
}

RTC::ReturnCode_t aliveLibIsAlive()
{
	RTC::ReturnCode_t	result = RTC::RTC_OK;

	// 生存データを拾って旧データと比較する
	// 5秒以上同一データとなっている場合は、監視対象が死んだものとして、監視対象を起動しなおす
	aliveFlagData.data = time(0);

	// 生存データの送信
	aliveOutPort.write();
	aliveFlagData.data = 0;

	// 生存データの取得
	aliveInPort.read();

	if(aliveOldData != aliveFlagData.data){
		// 対象は生きてる
		lastChangeTime = time(0);
		aliveOldData = aliveFlagData.data;
	}
	else{
		// 対象は・・・
		if((time(0) - lastChangeTime) > 5){
			// 沈黙しているみたいだ・・・
			result = RTC::PRECONDITION_NOT_MET;
			aliveLibRecover();
		}
	}
	return result;
}

RTC::ReturnCode_t aliveLibRecover()
{
	RTC::ReturnCode_t	result = RTC::RTC_OK;

	for(relativeCompIterator it = relativeComponents.begin(); it != relativeComponents.end(); it++){
		if(it->second.aliveTarget){
			runComponent(it->second.directoryName, it->second.fileName);
			break;
		}
	}

	return result;
}

RTC::ReturnCode_t aliveLibFinalize()
{
	RTC::ReturnCode_t	result = RTC::RTC_OK;

	return result;
}
