/*
 * RtcBatch.cpp
 *
 *  Created on: 2010/12/03
 *      Author: yotubata
 */

/**
 * @file autoLink.cpp
 * @brief RTコンポーネント群起動アプリケーション。
 * コンポーネント起動完了後にRTCリンク接続行い、コンポーネントのアクティブ化を行う。
 * @attension 本アプリケーションは各コンポーネント配下のrtc.confは参照せず、
 * 本アプリケーション下にあるrtc.confを使用してコンポーネントの検索・起動を行う。
 */
#define ENABLE_CLIENT_IR_SUPPORT
/*
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <pthread.h>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <string>
#include <vector>
#include <map>
#include <unistd.h>
#include <regex.h>

#include <rtm/CorbaNaming.h>
#include <rtm/RTObject.h>
#include <rtm/CorbaConsumer.h>
*/
#include <sys/wait.h>
#include <termios.h>
#include <dirent.h>

#include "Tools.h"
#include "RtcControl.h"
#include "RtcBatch.h"

/**
 * 機能拡張デファイン定義
 */
#define COMMAND_INPUT	1
#define DUMP_PORT_DATA	1

#if 0
/* 起動情報ファイルオリジナルデータ */
std::vector<std::string>	defList;
#endif
/* 起動情報設定ファイル内の情報を格納 */
def_file_table				iniFileMap;
/* コンポーネントリスト */
std::vector<std::string>	compList;
/* プロセスリスト */
component_table				compProcessList;
/* コマンドラインマップ */
command_line_table			commandlineMap;
/* コマンドリスト */
command_table				commandList;
/* コマンド履歴 */
std::vector<std::string>	commandHistory;
unsigned int				historyIndex = 0;
std::vector<std::string>	commandDoc;

/* 起動ディレクトリ退避 */
std::string	baseDir;
/* デフォルトrtc.confファイル名退避 */
std::string	defRtcConf = "";

/* コンポーネント候補 */
component_table				candidate_comp;

/* 起動パラメタ退避 */
#ifdef  _BACKUP_ARGS
int			system_argc;
char		**system_argv;
#endif

/* デバッグログ表示 */
#ifdef _DEBUG
void dumpIniFile();
void dumpMapInfo();
void dumpCompList();
void dumpCommandLine();
void dumpCmdLineInfo();
#endif

// 手抜きのために、ネームスペースの使用を宣言
using namespace std;
using namespace RTC;


/**
 * @brief ツールのメイン関数
 */
int main (int argc, char** argv)
{
	int			innerRet = 1;
	int			thread_ret;
	pthread_t	thread_id;

#ifdef  _BACKUP_ARGS
	/* 起動パラメタを退避する */
	{
		system_argc = argc;
		system_argv = argv;
	}
#endif

	// 自身の起動ディレクトリを取得する
	{
		char	lpszBaseDir[1024];
		char	*temp;
		/*
		FILE	*fd;
		int		readSize;

		memset(lpszBaseDir, 0, sizeof(lpszBaseDir));
		sprintf(lpszBaseDir, "/proc/%d/cmdline", getpid());

		fd = fopen(lpszBaseDir, "r");
		memset(lpszBaseDir, 0, sizeof(lpszBaseDir));
		readSize = fread(lpszBaseDir, sizeof(char), sizeof(lpszBaseDir), fd);
		fclose(fd);
		*/
		memset(lpszBaseDir, 0, sizeof(lpszBaseDir));
		temp = getcwd(lpszBaseDir, sizeof(lpszBaseDir));
		baseDir = lpszBaseDir;
	}

	// コマンドリストを生成する
	createCommandList();

	/* CorbaORB オブジェクトを取得する */
	corbaORB = CORBA::ORB_init(argc, argv);

	// とりあえずlocalhostは最初から参照しておく
	connectNamingServer("localhost");

	if(argc > 1){
		// 起動情報ファイル指定あり
		innerRet = readConfigurationFile(argc, argv);
	}

#if 0
	innerRet = readRtcConf();
	if(innerRet != -1){
		printf("readRtcConf\n");
		if(argc > 1){
			innerRet = readConnectInfoFile(argv[1]);
		}
		else{
			innerRet = readConnectInfoFile("default.rbdf");
		}
		printf("readConnectInfoFile\n");
	}
	else{
		printf("RTC Conf error! %d\n", innerRet);
	}
#endif
#if 0
	if(innerRet != -1){
		innerRet = createIniFileTable();
		innerRet = createCommandLineTable();
		innerRet = createComponentTable();
		innerRet = startComponent();
		innerRet = linkComponent();
//		innerRet = activateComponent();
	}
#endif

	if(innerRet > 0){
		/* スレッドを生成して、コマンド入力を受け付けれるようにする必要が生じた。
		 * ※非活性化を行わせる必要があるかも知れないので、コマンド受付を行う。
		 */
#ifdef COMMAND_INPUT
		thread_ret = pthread_create(&thread_id, NULL, commandInput, NULL);
#endif
		while(compProcessList.size()){
			int		status = 0;
			pid_t	endPid;
			std::string	pid;
			endPid = wait(&status);
			pid = "" + endPid;
			cout << compProcessList[pid] << "はステータス" << status << "で終了しました。" << endl;
			compProcessList.erase(pid);
			if(compProcessList.size() == 0){
				cout << "コンポーネントが全て終了しました。" << endl;
			}
		}

		/* スレッドの終了待機をする必要がある。
		 * スレッドに対して停止を指示する。
		 */
#ifdef COMMAND_INPUT
		thread_ret = pthread_join(thread_id, NULL);
#endif
	}
	return 0;
}

int readConfigurationFile(int argc, char** argv)
{
	int		result = 1;
	std::vector< std::string >	fileList;
#if 0
	int		ret;
	FILE	*fd;
#endif
	// 情報ファイルの形式を判別する必要がある
	for(int arg_cnt = 1; arg_cnt < argc; arg_cnt++){
		// 引数の数分ループして、設定ファイルを読み込む
		if(argv[arg_cnt][0] == '-'){
			// コマンドパラメタ
			if(strcmp(argv[arg_cnt], FULLAUTO_START_FLAG) == 0){
				if(argc != 2){
					cout << "完全自動実行指定時は「" << FULLAUTO_START_FLAG << "」フラグのみ指定する必要があります" << endl;
					result = -1;
				}
				// rtc.confの確認
				if(access(RTC_CONF_FILENAME, F_OK) != 0){
					cout << "完全自動実行に必要な「rtc.conf」ファイルが見つかりません" << endl;
					result = -1;
				}
				// default.brdfの確認
				if(access(DEFAULT_CONFIG_FILE_NAME, F_OK) != 0){
					cout << "完全自動実行に必要な「default.rbdf」ファイルが見つかりません" << endl;
					result = -1;
				}
				result = full_auto;
				break;
			}
			if(strcmp(argv[arg_cnt], PARTAUTO_START_FLAG) == 0){
				result = part_auto;
				continue;
			}
			if((strcmp(argv[arg_cnt], "-quit") == 0) || (strcmp(argv[arg_cnt], "-q")) == 0){
				result = 0;
			}
		}
		else{
			fileList.push_back(argv[arg_cnt]);
		}
#if 0
		fd = fopen(argv[arg_cnt], "r");
		if(fd != NULL){
			char	pBuff[512];
			int		readSize;

			// ファイルをてきとーに読み込む
			memset(pBuff, 0, sizeof(pBuff));
			readSize = fread(pBuff, sizeof(char), sizeof(pBuff), fd);
			fclose(fd);

			// ファイルの先頭付近にある、とあるデータを探す
			if(strcasestr(pBuff, RTC_NAMINGSRV_KEY) != NULL){
				if(defRtcConf.length() > 0){
					cout << "指定可能なrtc.confファイルを一つだけです" << endl;
					result = -1;
				}
				defRtcConf = argv[arg_cnt];
				continue;
			}

			if(strcasestr(pBuff, ORIGINAL_CONF_KEY) != NULL){
				ret = readConnectInfoFile(argv[arg_cnt]);
				result = 0;
			}
		}
		else{
			std::cout << "オープンエラー : " << argv[arg_cnt] << std::endl;
		}
#endif
	}
	if(fileList.size() > 0){
		// スクリプトファイルが指定されている
		// ファイルの存在を確認して、存在している場合は実行
		for(std::vector< std::string >::iterator it = fileList.begin(); it != fileList.end(); it++){
			if(access(it->c_str(), F_OK) == 0){
				run_command("run " + *it);
			}
			else{
				cout << "ファイルが見つかりません [ " << *it << " ]" << endl;
			}
		}
	}
	return result;
}

/**
 * @brief コマンド入力プロンプトの表示及びコマンド入力受付スレッド
 * @return スレッド終了ステータス
 */
void* commandInput(void* param)
{
	bool	app_end = false;
	int		read_len, status = 0;
	unsigned int	key_cnt;
	char	inBuff[1024];
	std::string	commandStr, fixCommand;
	struct termios term_attr, attr_back;

	// 端末属性を変更する
	tcgetattr(0, &term_attr);
	attr_back = term_attr;
	term_attr.c_cc[VMIN] = 1;
	term_attr.c_cc[VTIME] = 0;
	term_attr.c_lflag = ~(ICANON | ECHO);
//	term_attr.c_lflag = ~(ICANON);
	tcsetattr(0, TCSANOW, &term_attr);

	/* おとなしく入力待機を行う。 */
	while(!app_end){
		bool	escFlag = false, cursorFlag = false, delKey = false, homeEndFlag = false;
		memset(inBuff, 0, sizeof(inBuff));
		std::cout << "command => ";

		if(fixCommand.length() > 0){
			strcpy(inBuff, fixCommand.c_str());
			key_cnt = fixCommand.length();
			printf("%s", inBuff);
			fixCommand = "";
		}
		else{
			key_cnt = 0;
		}
		fflush(stdout);
		/*
		 * コマンドを受け取る
		 * 0x0a : 改行
		 * 0x09 : Tab
		 * 0x7f : BackSpace
		 * 0x1b 5b 33 7e : Delete
		 * 0x1b : Esc
		 * 0x1b 5b 41 : ↑
		 * 0x1b 5b 42 : ↓
		 * 0x1b 5b 43 : →
		 * 0x1b 5b 44 : ←
		 * 0x1b 4f 48 : Home
		 * 0x1b 4f 46 : End
		 */
#if 1
//		key_cnt = 0;
		while(1){
			read_len = read(0, &status, 1);
//			printf("文字コード = %02x\n", status);
			if(delKey){
				delKey = false;
				escFlag = false;
				cursorFlag = false;
				printf("%c", (char)(0x0e));
				fflush(stdout);
				continue;
			}
			if(homeEndFlag){
				escFlag = false;
				homeEndFlag = false;
				switch(status){
					case 0x48:	// Home
						break;
					case 0x46:	// End
						break;
				}
				continue;
			}
			if(cursorFlag){
				switch(status){
					case 0x41:
						// コマンド履歴を遡る
						if(historyIndex > 0){
							historyIndex--;
							// カーソルを左に移動
							if(key_cnt > 0){
								printf("\x1b[%dD", key_cnt);
							}
							// 行末まで削除
							printf("\x1b[K");
							// 履歴コマンド挿入
							std::cout << commandHistory[historyIndex];
							memset(inBuff, 0, sizeof(inBuff));
							strcpy(inBuff, commandHistory[historyIndex].c_str());
							key_cnt = commandHistory[historyIndex].length();
							// 再表示
							fflush(stdout);
						}
						break;
					case 0x42:
						if(commandHistory.size() > (historyIndex + 1)){
							historyIndex++;
							// カーソルを左に移動
							if(key_cnt > 0){
								printf("\x1b[%dD", key_cnt);
							}
							// 行末まで削除
							printf("\x1b[K");
							// 履歴コマンド挿入
							std::cout << commandHistory[historyIndex];
							memset(inBuff, 0, sizeof(inBuff));
							strcpy(inBuff, commandHistory[historyIndex].c_str());
							key_cnt = commandHistory[historyIndex].length();
							// 再表示
							fflush(stdout);
						}
						else{
							if(key_cnt > 0){
								// カーソルを左に移動
								printf("\x1b[%dD", key_cnt);
							}
							// 行末まで削除
							printf("\x1b[K");
							memset(inBuff, 0, sizeof(inBuff));
							key_cnt = 0;
							fflush(stdout);
						}
						break;
					case 0x43:
						if((key_cnt) < strlen(inBuff)){
							key_cnt++;
							printf("\x1b[1C");
							fflush(stdout);
						}
						break;
					case 0x44:
						if(key_cnt > 0){
							key_cnt--;
							printf("\x1b[1D");
							fflush(stdout);
						}
						break;
					case 0x33:		// Del
						delKey = true;
						break;
				}
				escFlag = false;
				cursorFlag = false;
				continue;
			}
			if(escFlag){
				// エスケープ中
				switch(status){
					case 0x5b:
						// カーソルコントロールの可能性あり
						cursorFlag = true;
						continue;
					case 0x4f:
						// home / end キー
						homeEndFlag = true;
						continue;
					default:
						escFlag		= false;
						cursorFlag	= false;
						homeEndFlag	= false;
						break;
				}
			}
			switch(status){
				case 0x7f:	// Back spaceキー
					if(key_cnt > 0){
						key_cnt--;
						printf("%c %c", (char)(0x08), (char)(0x08));
						fflush(stdout);
						inBuff[key_cnt] = 0x00;
					}
					break;
				case 0x0a:	// 改行コード
					printf("%c", (char)status);
					fflush(stdout);
					break;
				case 0x1b:	// エスケープコード
					escFlag = true;
					break;
				case 0x09:	// Tabキー
					// 入力文字列候補表示を行う
					fixCommand = inputSupport(inBuff);
					break;
				default:
					if(key_cnt == strlen(inBuff)){
						// 末尾に追加
						inBuff[key_cnt] = (char)status;
						printf("%c", (char)status);
					}
					else{
						memmove(&(inBuff[key_cnt + 1]), &(inBuff[key_cnt]), (strlen(inBuff) - key_cnt));
						inBuff[key_cnt] = (char)status;
						printf("%s", &(inBuff[key_cnt]));
						printf("\x1b[%dD", strlen(inBuff) - key_cnt -1);
					}
					fflush(stdout);
					key_cnt++;
					break;
			}
			if(status == 0x09) break;
			if(status == 0x0a) break;

			continue;
		}
#else
		key_cnt = 0;
		while(1){
			status = getchar();
			if(status == 0x0a)	break;
			inBuff[key_cnt] = (char)status;
			key_cnt++;
		}
#endif
		if(status == 0x0a){
			// コマンド実行
			switch(run_command(inBuff)){
				case 1:
					commandHistory.push_back(inBuff);
					historyIndex = commandHistory.size();
					break;
				case -1:
					break;
				case 0:
					app_end = true;
					break;
			}
		}
	}
	tcsetattr(0, TCSANOW, &attr_back);
	return 0;
}

int run_command(const std::string command)
{
	std::string commandStr = command;
	std::vector<std::string> cmd_str;
	command_table::iterator	comd_it;

	if(commandStr.length() == 0){
		return -1;
	}
	// コマンド分解（コマンド部とパラメタ部に分解する）
	cmd_str = string_token(commandStr, " ");
	if(cmd_str.size() > 0){
		commandStr = cmd_str[0];
	}
	else{
		commandStr = "";
	}

	// コマンド存在確認
	comd_it = commandList.find(commandStr);
	if(comd_it != commandList.end()){
		// 関数登録済みチェック
		if(comd_it->second.func != 0){
			// 関数呼び出し
			return comd_it->second.func(commandStr, &cmd_str);
		}
	}
	else{
		// コマンドが存在しない
		std::cout << "\n" << commandStr << ": command not found\n" << std::endl;
	}

	return -1;
}

/**
 * @brief 入力補完リストの表示を行う
 * @param buff 既入力文字列
 */
std::string inputSupport(const char *buff)
{
	std::string					result = buff;
	std::vector<std::string>	candidateList;
	std::vector<std::string>	candidate_str;
	command_table::iterator		it;

	std::cout << "" << std::endl;

	// 既入力文字列から入力候補の表示を行う
	if(strlen(buff) == 0){
		// 既入力文字列がないので、コマンド一覧の表示を行う
		std::cout << "" << endl;
		for(it = commandList.begin(); it != commandList.end(); it++){
			if(!(it->second.alias)){
				std::cout << "\t" << it->first << std::endl;
			}
		}
	}
	else{
		std::vector<std::string> cmd_str;
		std::vector<std::string>::iterator	it;

		// コマンド・第1引数・第2引数に分解する
		cmd_str = string_token(buff, " ");
		switch(cmd_str.size()){
			case 0:
				candidate_str = candidateCommand();
				break;
			case 1:		// コマンド部推定 / ネーミングサーバ部推定
				// 既にスペースが入ってるか確認する
				if(buff[strlen(buff) -1] == ' '){
					// 空白がある場合はコマンドが成立しているとみなす
					candidate_str.clear();
					candidate_str.push_back(cmd_str[0]);
				}
				else{
					candidate_str = candidateCommand(cmd_str[0]);

					if(candidate_str.size() == 1){
						// 何かが確定した
						result = candidate_str[0] + " ";
						// コマンド用補完候補検索へ移行
					}
					else{
						unsigned int	matchCnt;
						std::string	firstStr = "", secondStr = "";

						for(it = candidate_str.begin(); it != candidate_str.end(); it++){
							if((firstStr.length() == 0) && (it == candidate_str.begin()))
								firstStr = *it;
							secondStr= *it;
							cout << "\t" << *it << endl;
						}

						for(matchCnt = result.length(); matchCnt <= firstStr.length() ; matchCnt++){
							if(strncmp(firstStr.c_str(), secondStr.c_str(), matchCnt) != 0){
								break;
							}
						}
						result = firstStr.substr(0,matchCnt - 1);
						if(result.length() == 0){
							result = buff;
						}
					}

					break;
				}
			case 2:		// ネーミングサーバー部推定 / コンポーネント部推定
			default:	// 3個以上・・・コマンド　第1パラメタ　第2パラメタ・・・・
				// コマンド情報を取り出す
				{
					std::string			cmd;
					command_table::iterator	comd_it;
					ShowCandidateProc		func = candidateComponent;

					// コマンド存在確認
					comd_it = commandList.find(cmd_str[0]);
					if(comd_it != commandList.end()){
						// 関数登録済みチェック
						func = (comd_it->second.showCandidate != NULL) ? comd_it->second.showCandidate : func;
					}
					if((cmd_str.size() == 1) || (buff[strlen(buff) -1] == ' ')){
						cmd_str.push_back("");
					}
#if 0
					if((cmd_str.size() >= 2) && (buff[strlen(buff) -1] == ' ')){
						cmd_str.push_back();
					}
#endif
					// とりあえず今は補完機能を終わらせよう
					if(func != candidateComponentConfig)
						cmd_str[cmd_str.size() - 1] = func(cmd_str[cmd_str.size() - 1]);
					else{
						cmd_str[cmd_str.size() - 1] = func(buff);
					}

					result = "";
					for( it = cmd_str.begin(); it != cmd_str.end(); it++){
						if(result.length() > 0) result = result + " ";
						result = result + *it;
					}
				}
				break;
		}
	}

	return result;
}

std::string candidateVoid(const std::string command)
{
	std::string	result = command;

	return result;
}

std::string candidateNaming(const std::string command)
{
	return candidateComponentAndServer(command, CALLBACK_TYPE_SERV_ALL);
}

std::string candidateFile(const std::string command)
{
	DIR	*dd;
	unsigned int	tabCnt, cnt = 0, nameLen = 0, col = 0, nameWidth;
	struct dirent *dirent;
	command_line_table	fileList;
	std::string	result = command;
	std::string	refFolder = baseDir;
	std::string tempdirName = "";
	std::string fileName = command;
	std::string findName;

// cout << command << endl;

	// baseDir
	/*if(command.find_first_of("/") != std::string::npos)*/{
		std::vector< std::string > dirnameTree;

		// フォルダ指定されている
		if(command[0] == '/'){
			// ルートからの絶対
			refFolder = "";
		}
		else{
			// カレントフォルダからの相対
			refFolder = refFolder + "/";
		}

		dirnameTree = string_token(command, "/");
		if(command[command.length()-1] == '/'){
			for(std::vector< std::string >::iterator it = dirnameTree.begin(); it != dirnameTree.end(); it++){
				refFolder = refFolder + *it + "/";
			}
		}
		else{
			for(std::vector< std::string >::iterator it = dirnameTree.begin(); it != dirnameTree.end(); it++){
				if(tempdirName.length() > 0)
					refFolder = refFolder + tempdirName + "/";
				tempdirName = *it;
			}
		}

		fileName = tempdirName;
	}
//cout << "Folder Name = " << refFolder << endl;
	if(refFolder[0] != '/')	refFolder = "/" + refFolder;

	if((dd = opendir(refFolder.c_str())) == NULL)
		return result;

	while((dirent = readdir(dd)) != NULL){
		findName = dirent->d_name;
		// 候補文字列チェック
		if((fileName.length() == 0) || (findName.find(fileName) == 0)){
			// ダイレクト出力ではなく、vectorバッファに格納しつつ最大文字数を取得する
			if(dirent->d_type & DT_DIR)	findName = findName + "/";
			fileList.insert(command_line(findName, findName));
			if(nameLen < findName.length())	nameLen = findName.length();
		}
	}
	// コンソールの幅を取得する
	// 現在は８０固定,タブは８固定
	// コンソールに出力できる列数を計算する
	nameWidth = (nameLen + 8) / 8 * 8;
	col = 80 / nameWidth;
	cnt = 0;

//cout << "MaxLen = " << nameWidth << endl;
	if(fileList.size() == 1){
		// 確定
		result = fileList.begin()->first;
	}
	else{
		std::string		firstName, lastName;
		unsigned int matchCnt;

		// リスト表示
		for(component_table::iterator it = fileList.begin(); it != fileList.end(); it++){
			cnt++;
			cout << it->first;

			lastName = it->first;
			for(tabCnt = 0; tabCnt < ((nameWidth - it->first.length() + 7) / 8); tabCnt++){
				cout << "\t";
			}

			if(cnt == col){
				cout << "\n";
				cnt = 0;
			}
		}
		cout << endl;

		if(fileList.size() > 0){
			firstName = fileList.begin()->first;
			for(matchCnt = fileName.length(); matchCnt <= firstName.length() ; matchCnt++){
				if(strncmp(firstName.c_str(), lastName.c_str(), matchCnt) != 0){
					break;
				}
			}
			result = firstName.substr(0,matchCnt - 1);
		}
	}
	closedir(dd);

	return refFolder + result;
}

std::string candidateComponent(const std::string command)
{
	return candidateComponentAndServer(command, CALLBACK_TYPE_ALL);
}

std::string candidateComponentPort(const std::string command)
{
	// コンポーネント@ポート コンポーネント@ポート
	std::string			result = command;
	std::string			compName;
	std::string			inputName = "";
	CORBA::Object_ptr	pComp;
	std::vector< std::string >	param;

	//cout << command << "[Term" << endl;
	// コンポーネントが確定しているか確認する
	if(command.find_last_of('@') != string::npos){
		// ポートを列挙する
		param = string_token(command, " @");
		if(param.size() > 2)
			compName = param[2];
		else
			compName = param[0];
		pComp = findComponent(compName);
		if(pComp != NULL){
			// ポート名列挙
			// ポート情報を取得する
			CorbaConsumer<RTC::RTObject>	targetComp;
			PortServiceList					*pTargetPorts;
			PortProfile						*portProfile;
			int								portCnt, portMax;
			unsigned int					matchCnt;
			SDOPackage::NVList				propListNV;
			RTC::Properties					portProp;
			std::string						portName, firstName, lastName;
			ConnectorProfileList			connectList;
			ComponentProfile				*compProfile;

			component_table					port_list;

			targetComp.setObject(pComp);
			if(param.size() > 1)
				inputName = param[1];
			if(param.size() > 3)
				inputName = param[3];

			// インスタンス名も出力しておく
			compProfile = targetComp->get_component_profile();
			pTargetPorts = targetComp->get_ports();
			portMax = (int)(pTargetPorts->length());
			for(portCnt = 0; portCnt < portMax; portCnt++){
				portProfile = ((*pTargetPorts)[portCnt])->get_port_profile();
				// ポート名取得
				portName = portProfile->name;
				if((portName.find(inputName) == 0) || (inputName.length() == 0)){
					port_list[portName] = portName;
				}
			}
			if(port_list.size() == 1){
				// 確定
				result = port_list.begin()->first;
			}
			else if(port_list.size() > 0){
				// リスト表示
				firstName = port_list.begin()->first;
				for(component_table::iterator it = port_list.begin(); it != port_list.end(); it++){
					lastName = it->first;
					cout << "\t" << it->second << endl;
				}

				for(matchCnt = inputName.length(); matchCnt <= firstName.length() ; matchCnt++){
					if(strncmp(firstName.c_str(), lastName.c_str(), matchCnt) != 0){
						break;
					}
				}
				result = firstName.substr(0,matchCnt - 1);
			}
			if(result.length() == 0){
				result = command;
			}
			else{
				result = compName + "@" + result;
			}
		}
	}
	else{
		// コンポーネント特定中
		result = candidateComponent(command);
	}

	return result;
}

std::string candidateComponentConfig(const std::string command)
{
	// コマンド コンポーネント コンフィグ=値 コンフィグ=値
	std::string					result = "";
	std::string					newCommand = "";
	std::string					inputName = "";
	CORBA::Object_ptr			pComp = NULL;
	std::vector< std::string >	param;

cout << command << "[Term"  << endl;

	// まずは、コマンドとパラメタに分解する
	param = string_token(command, " ");
	if(param.size() > 2){
		// コンポーネントが確定して、コンフィグの一部があるいは全部入っている
		pComp = findComponent(param[1]);

		// 一度コマンドを組立て直す
		for(unsigned int cnt = 2; cnt < param.size(); cnt++){
			newCommand = newCommand + param[cnt] + " ";
		}
		param = string_token(newCommand, " =");

		if((param.size() % 2) != 0){
			// コンフィグ名で止まってる
			inputName = param[param.size() - 1];
		}
		else if(command[command.length() -1] == ' '){
			// 次のコンフィグを探す状態
			inputName = "";
		}
		else{
			// コンフィグ値入力中
			return result;
		}
	}
	else{
		// コンポーネントが確定しているかも？
		// コンポーネント特定中
		if(param.size() == 1){
			result = candidateComponent("");
		} else if(command[command.length() -1] == ' '){
			// コンポーネントが確定しているみたいだ
			pComp = findComponent(param[1]);
			inputName = "";
		}
		else{
			// コンポーネント探索中と解釈する
			result = candidateComponent(param[1]);
		}
		if(pComp == NULL)	return result;
	}


	if(pComp == NULL)	return result;


	if(pComp != NULL){
		// ポート名列挙
		// ポート情報を取得する
		CorbaConsumer<RTC::RTObject>	targetComp;
		unsigned int					matchCnt;
		SDOPackage::NVList				propListNV;
		RTC::Properties					portProp;
		std::string						portName, firstName, lastName;
		ConnectorProfileList			connectList;
		std::string						curConfName;
		SDOPackage::Configuration_ptr	config;
		RTC::Properties					confProp;
		SDOPackage::ConfigurationSet	*configValue;

		component_table					port_list;

		targetComp.setObject(pComp);

#if 1
		config = targetComp->get_configuration();
		configValue = config->get_active_configuration_set();
		propListNV = configValue->configuration_data;
		confProp = NVUtil::toProperties(propListNV);
		for(unsigned int n = 0 ; n < propListNV.length(); n++){
//				std::cout << "\t" << propListNV[n].name << "\t: " << confProp[(char*)(propListNV[n].name)] << std::endl;
			portName = propListNV[n].name;
			if((portName.find(inputName) == 0) || (inputName.length() == 0)){
				port_list[portName] = portName;
			}
		}
#else
		for(portCnt = 0; portCnt < portMax; portCnt++){
			portProfile = ((*pTargetPorts)[portCnt])->get_port_profile();
			// ポート名取得
			portName = portProfile->name;
			if((portName.find(inputName) == 0) || (inputName.length() == 0)){
				port_list[portName] = portName;
			}
		}
#endif
		if(port_list.size() == 1){
			// 確定
			result = port_list.begin()->first;
		}
		else if(port_list.size() > 0){
			// リスト表示
			firstName = port_list.begin()->first;
			for(component_table::iterator it = port_list.begin(); it != port_list.end(); it++){
				lastName = it->first;
				cout << "\t" << it->second << endl;
			}

			for(matchCnt = inputName.length(); matchCnt <= firstName.length() ; matchCnt++){
				if(strncmp(firstName.c_str(), lastName.c_str(), matchCnt) != 0){
					break;
				}
			}
			result = firstName.substr(0,matchCnt - 1);
		}
/*
		if(result.length() == 0){
			result = command;
		}
		else{
			result = compName + " " + result;
		}
*/
	}
/*
	else{
		// コンポーネント特定中
		result = candidateComponent(command);
	}
*/

	return result;
}

std::string candidateComponentAndServer(const std::string command, const int type)
{
	std::string		result = command;
	unsigned int	matchCnt;
	std::string	firstStr = "", secondStr = "";

	findCompName = command;
	candidate_comp.clear();
//	enumNamingServers(listShowComponentCallback, CALLBACK_TYPE_COMP);
	enumNamingServers(listShowComponentCallback, type);

	if(command.length() > 0){
		// 候補の表示??
		if(candidate_comp.size() == 1){
			// 確定
			firstStr = candidate_comp.begin()->first;
			matchCnt = firstStr.length() + 1;
		}
		else{
			for(component_table::iterator it = candidate_comp.begin(); it != candidate_comp.end(); it++){
				if(it->first.find(command) == 0){
					if((firstStr.length() == 0) && (it == candidate_comp.begin()))
						firstStr = it->first;
					secondStr = it->first;
					cout << "\t" << it->second << endl;
				}
			}

			for(matchCnt = result.length(); matchCnt <= firstStr.length() ; matchCnt++){
				if(strncmp(firstStr.c_str(), secondStr.c_str(), matchCnt) != 0){
					break;
				}
			}
		}
		result = firstStr.substr(0,matchCnt - 1);
		if(result.length() == 0){
			result = command;
		}
	}
	else{
		for(component_table::iterator it = candidate_comp.begin(); it != candidate_comp.end(); it++){
			cout << "\t" << it->second << endl;
		}
		result = "";
	}

	return result;
}
void listShowComponentCallback(std::string name, std::string kind, int type, int layered, std::string path, CORBA::Object_ptr pComp)
{
	std::string addStr1 = "";
	std::string addStr2 = "";

	if(type != 0){
		if(type & (CALLBACK_TYPE_CNTX | CALLBACK_TYPE_COMP)){
			//std::cout << "\t" << path << "/" << name << std::endl;
			addStr1 = path + "/" + name;
		}
		if(type & (CALLBACK_TYPE_COMP | CALLBACK_TYPE_SERV)){
			//std::cout << "\t" << name << std::endl;
			addStr2 = name;
		}
	}

	if(addStr1.find(findCompName) == 0){
		candidate_comp[addStr1] = addStr1;
	}
	if(addStr2.find(findCompName) == 0){
		candidate_comp[addStr2] = addStr2;
	}
}


std::vector<std::string> candidateCommand(const std::string command)
{
	std::vector<std::string> result;
	command_table::iterator		it;

	for(it = commandList.begin(); it != commandList.end(); it++){
		if(it->first.find(command) == 0){
			if(!(it->second.alias)){
				result.push_back(it->first);
			}
		}
	}
	return result;
}

#if 0
/**
 * @brief 設定ファイル内容をハッシュテーブルに格納する
 * @return 格納結果
 */
int createIniFileTable()
{
	std::string::size_type sec_start, sec_end;
	std::vector< std::string >::iterator	it;
	std::vector< std::string >	sectionData;
	std::string				hash_name;

	hash_name = "";
	for(it = defList.begin(); it != defList.end(); it++){
		// []セクション宣言を探す
		// 前提条件：行頭に存在している
		sec_start = it->find("[");
		sec_end = it->find("]");

		if((sec_start != std::string::npos) &&
			(sec_end != std::string::npos)){
			// 新セクションを処理する前に直前データを退避する
			if(hash_name.length() > 0){
				// 取得したセクション内データをキーと関連付ける
				iniFileMap.insert(table_val(hash_name, sectionData));
				sectionData.clear();
			}

			// [section]データ
			// ハッシュテーブルインデックスを作成する
			hash_name = it->substr(0, sec_end);
			hash_name = hash_name.substr(sec_start + 1, hash_name.length() - sec_start);
		}
		else{
			// 普通の行
			sectionData.push_back(*it);
		}
	}
	return 0;
}
#endif

#if 0
/**
 * @brief rtc.confの読み込みを行う
 * @return 読み込み結果
 */
int readRtcConf(const std::string rtc_conf)
{
	char		lineData[1024];
	int			ret = 0;
	int			line = 0;
	std::string	data;
	std::string::size_type index;

	// ファイルストリームを作成する
	ifstream ifs(rtc_conf.c_str(), ios::in);
	if(ifs){
		for(line = 0; !ifs.eof(); line++){
			memset(lineData, 0, sizeof(lineData));
			ifs.getline(lineData, sizeof(lineData));
			data = lineData;
			cout << data << endl;
			if(data.length() > 1){
				// rtc.confを解析してcorba.nameserversを取り出す
				index = data.find(RTC_NAMINGSRV_KEY);
				if(index != std::string::npos){
					std::vector<std::string>	paramList;

					data = data.substr(index + strlen(RTC_NAMINGSRV_KEY),
										data.length() - index - strlen(RTC_NAMINGSRV_KEY));
					connectNamingServer(data);
					break;
				}
			}
		}
	}
	else{
		ret = -1;		// ファイルが存在しなかった
	}
	return ret;
}
#endif

#if 0
/**
 * @brief 接続設定情報ファイルの読み込みを行う
 * @return 読み込み結果
 */
int readConnectInfoFile(const std::string fileName)
{
	char		lineData[1024];
	int			ret = 0;
	int			line = 0;
	std::string	data;
	std::string::size_type index;

	// ファイルストリームを作成する
	ifstream ifs(fileName.c_str(), ios::in);
	if(ifs){
		for(line = 0; !ifs.eof(); line++){
			memset(lineData, 0, sizeof(lineData));
			ifs.getline(lineData, sizeof(lineData));
			data = lineData;
			if(data.length() > 1){
				// コメント行は削除する
				index = data.find(";");
				if(index != std::string::npos){
					data = data.substr(0, index);
				}
				index = data.find("#");
				if(index != std::string::npos){
					data = data.substr(0, index);
				}
				if(data.length() > 1){
					// 空白行は削除する
					defList.push_back(data);
				}
			}
		}
		// 後の処理を簡単にするためにダミーのセクションを最終行に入れる
		defList.push_back("[]");
	}
	else{
		ret = -1;		// ファイルが存在しなかった
	}
	return ret;
}
#endif

/**
 * @brief コンポーネント起動のコマンドラインテーブルを作成する
 * @return テーブル作成結果
 */
int createCommandLineTable()
{
	std::string	commandLine;
	std::vector<std::string>	paramList, paramInfo;
	std::vector<std::string>::iterator	it;

	// セクション[ComponentList]を取得する
	if(iniFileMap[COMPONENT_LIST].size() > 0){
		paramList = iniFileMap[COMPONENT_LIST];
		for(it = paramList.begin(); it != paramList.end(); it++){
			commandLine = *it;
			paramInfo = commandLine_token(commandLine);
			if(paramInfo.size() == 2){
				commandlineMap.insert(command_line(paramInfo[0], paramInfo[1]));
			}
		}
	}
	return 0;
}

/**
 * @brief コンポーネントリストテーブルを作成する
 * @return テーブル作成結果
 */
int createComponentTable()
{
	bool				bFind;
	std::vector<std::string>	compNames, connectList;
	std::vector<std::string>::iterator	it, compIt;
	std::string				lastComponent;

	// 最終起動コンポーネント名を取得する
	if(iniFileMap[START_COMP_SECTION].size() > 0){
		lastComponent = iniFileMap[START_COMP_SECTION][0];
	}
	if(iniFileMap[LINK_SECTION].size() < 1)		return -1;
	connectList = iniFileMap[LINK_SECTION];

	for(it = connectList.begin(); it != connectList.end(); it++){
		/*
		 * コンポーネント名：ポート名 = コンポーネント名：ポート名
		 * となっているので、1行で２つのコンポーネントを入手できる
		 */
		// " =:\r\n"でパースして、[0][2]を取得してリストに格納
		compNames = string_token(*it, " =:\r\n");
		// すでにリスト上に存在しているか確認し、存在していない場合は格納する
		for(size_t cnt = 0; cnt < compNames.size(); cnt = cnt + 2){
			bFind = false;
			if(lastComponent == compNames[cnt])	continue;
			for(compIt = compList.begin(); compIt != compList.end(); compIt++){
				if(*compIt == compNames[cnt]){
					bFind = true;
					break;
				}
			}
			if(!bFind){
				compList.push_back(compNames[cnt]);
			}
		}
	}
	if(lastComponent.length() > 0){
		compList.push_back(lastComponent);
	}
	return 0;
}

#if 0
void listupComponent(const std::string namingServerName)
{
	std::cout << "" << endl;
	corbaNamingList::iterator		itNamingServer;

	for(itNamingServer = namingServerList.begin(); itNamingServer != namingServerList.end(); itNamingServer++){
		if((namingServerName.length() == 0) || (itNamingServer->first == namingServerName)){
			// ネーミングサーバー名表示
			std::cout << itNamingServer->first << std::endl;
			componentList(itNamingServer->second, itNamingServer->second.getRootContext());
			std::cout << "" << endl;
		}
	}
}

void componentList(RTC::CorbaNaming curNaming, CosNaming::NamingContext_ptr curContext, int nest_cnt)
{
	CosNaming::NamingContext_ptr	nextContext;
	CosNaming::BindingList_var		bindList;
	CosNaming::BindingIterator_var	bindListIt;
	CORBA::Object_ptr				pComp = NULL;

	nest_cnt++;

	if(curContext == 0)		return;
	curNaming.list(curContext, 10, bindList, bindListIt);
	for(size_t bindCnt = 0; bindCnt < bindList->length(); bindCnt++){
		for(size_t nameCnt = 0; nameCnt < bindList[bindCnt].binding_name.length(); nameCnt++){
			for(int cnt = 0; cnt < nest_cnt; cnt++)
				std::cout << "\t";
#if 1
//			if(strcmp(bindList[bindCnt].binding_name[nameCnt].kind, "rtc") == 0){
			if(bindList[bindCnt].binding_type == CosNaming::nobject){
				pComp = findComponent((char *)(bindList[bindCnt].binding_name[nameCnt].id), (char *)(bindList[bindCnt].binding_name[nameCnt].kind), curNaming.getNameServer());
				// コンポーネントが見つかった
				if(pComp != NULL){
					try{
						if(pComp->_non_existent())
							cout << bindList[bindCnt].binding_name[nameCnt].id << "\t(!! zombie !!)" << endl;
						else
							cout << bindList[bindCnt].binding_name[nameCnt].id << "\t(" << bindList[bindCnt].binding_name[nameCnt].kind << ")" << endl;
					}
					catch(...){
						cout << bindList[bindCnt].binding_name[nameCnt].id << "\t( unknown )" << endl;
					}
				}
				else{
					cout << (char *)(bindList[bindCnt].binding_name[nameCnt].id) << " コンポーネントが・・・" << endl;
				}
			}
			else{
				cout << bindList[bindCnt].binding_name[nameCnt].id << "\t(" << bindList[bindCnt].binding_name[nameCnt].kind << ")" << endl;
			}
#else
			cout << bindList[bindCnt].binding_name[nameCnt].id << "\t(" << bindList[bindCnt].binding_name[nameCnt].kind << ")" << endl;
#endif
			//cout << "kind = " << bindList[bindCnt].binding_name[nameCnt].kind << endl;
			/*
			if((strcmp(bindList[bindCnt].binding_name[nameCnt].kind, "rtc") != 0) &&
				(strcmp(bindList[bindCnt].binding_name[nameCnt].kind, "") != 0)){
				*/
			if(bindList[bindCnt].binding_type == CosNaming::ncontext){
				// ホストコンテキスト
				nextContext = curNaming.bindOrResolveContext(curContext, bindList[bindCnt].binding_name, curNaming.newContext());
				if(nextContext != NULL){
					componentList(curNaming, nextContext, nest_cnt);
				}
			}
		}
	}
}
#endif
#if 0
/**
 * @brief コンポーネント群の起動を行う
 * @return 起動結果
 */
int startComponent()
{
	std::vector<std::string>::iterator	it;
	std::string				lastComponent;

	for(it = compList.begin(); it != compList.end(); it++){
		// コンポーネント起動
		// 起動する前にCORBAサーバに登録されていないことを確認する
		CORBA::Object_ptr pComp;
		std::string compName;

		compName = CREATE_RTC_NAME(*it);
		pComp = findComponent(compName.c_str());
		if(pComp != NULL){
			// コンポーネントが見つかった
			if(pComp->_non_existent())
				cout << "オブジェクトチェック　真" << endl;
			else
				cout << "オブジェクトチェック　偽" << endl;
			cout << "コンポーネント：" << compName << " 起動済み" << endl;
			continue;
		}
		else{
			pid_t	cpid;
			int		pipe_fd[2];
			std::string	compFile, cpidStr;

			compFile = *it;

			if (pipe(pipe_fd) == -1) {
				// パイプ作成エラー
			}
			/*
			 * コンポーネントが登録されていないので起動する
			 * 起動には・・・自分を子プロセスで作動させて、プロセスイメージを変える
			 * 子プロセスのプロセスIDとコンポーネント名を退避させておく
			 */
			cpid = fork();
			if(cpid == -1){
				cout << "自分の起動に失敗した" << endl;
			}
			cpidStr = "" + cpid;
			compProcessList.insert(comp_val(cpidStr, compFile));
			if(cpid == 0){
				int				cnt, sys_ret;
				ssize_t	readSize = 1, totalSize = 0;
				size_t	commandLineLen = 0;
				char	*lpszCommandLine = 0;

				// 子プロセスが作動した
				// パイプ読み込み待機
				// パイプから取得した文字列でプロセスを起動する
				close(pipe_fd[1]);
				cnt = 0;
				while(readSize > 0){
					commandLineLen = DEF_BUFF_SIZE * (cnt + 1);
					lpszCommandLine = (char*)realloc(lpszCommandLine, commandLineLen);
					readSize = read(pipe_fd[0], &(lpszCommandLine[totalSize]), commandLineLen);
					totalSize = totalSize + readSize;
					cnt++;
				}
				close(pipe_fd[0]);

				// コマンドライン文字列をパースして、コマンド名とパラメタに分解する
cout << "CommandLine = " << lpszCommandLine << endl;
				{
					std::string	strCommandLine(lpszCommandLine);

					sys_ret = system(strCommandLine.c_str());
					free(lpszCommandLine);
					exit(EXIT_SUCCESS);
				}
			}
			else{
				size_t		writeLen;
				std::string	commandLine;
				char	*lpszCommandLine = 0;

				// 継続処理・・・パイプに起動するコンポーネントとパラメタを流す
				commandLine = *it;
				commandLine = commandlineMap[commandLine];
				if(commandLine.length() == 0){
					commandLine = *it;
				}
cout << "WriteCommand = " << commandLine << endl;
				lpszCommandLine = (char*)realloc(lpszCommandLine, commandLine.length() + 2);
				memset(lpszCommandLine, 0, commandLine.length() + 2);
				strcpy(lpszCommandLine,commandLine.c_str());

				close(pipe_fd[0]);
				writeLen = write(pipe_fd[1], lpszCommandLine, commandLine.length() + 2);

				close(pipe_fd[1]);
			}
		}
	}
	// コンポーネントの起動が完全に完了するのに時間がかかるので２秒ほど待機する
	sleep(2);
	return 0;
}
#endif
#if 0
#ifdef RTM_Ver4
PortList_var	findComponentPort(std::string compPortName, int portType)
{
	PortList_var result;
}
#else
PortService findComponentPort(std::string compPortName, int portType)
{
	PortService			result;
	PortServiceList		*portService = NULL;

	std::string			portTypeStr;

	std::vector<std::string>	compNames;
	std::string					compName, portName;
	CORBA::Object_ptr			pComp;

	// 接続元コンポーネント名をインスタンス名に変更する？？
	compNames	= string_token(compPortName, " =:\r\n");
	compName	= CREATE_RTC_NAME(compNames[0]);
	portName	= compNames[1];

	switch(portType){
		case 1:		// In
			portTypeStr = "DataInPort";
			break;
		case 2:		// Out
			portTypeStr = "DataOutPort";
			break;
		case 3:		// Corba
			portTypeStr = "CorbaPort";
			break;
	}

	// コンポーネントの検索を行う
	pComp		= findComponent(compName.c_str());
	if(pComp != NULL){
		CorbaConsumer<RTC::RTObject>	comp;


		int					ports_cnt;
		std::string			curPort, profName;
		std::string			srcDataType, destDataType, portType;
		SDOPackage::NVList	propListNV;
		RTC::Properties		portProp;
		PortProfile			*portProfile;

		comp.setObject(pComp);
		portService = comp->get_ports();
		ports_cnt = (int)(portService->length());

		for(int port_cnt = 0; port_cnt < ports_cnt; port_cnt++){
			result = ((*portService)[port_cnt]);
			portProfile = result.get_port_profile();

			/* ポートのプロパティを取得 */
			propListNV = portProfile->properties;
			portProp = NVUtil::toProperties(propListNV);
			portType = portProp["port.port_type"];

			// ポートタイプ判別
			if(portType == portTypeStr){
				// ポート名取得
				if(portName.compare(portProfile->name) == 0){
					// ポートが見つかった
					printf("ポートが見つかりました [%d]\n", port_cnt);
					/*
					if(portType != "CorbaPort")
						destDataType = portProp["dataport.data_type"];
					else
						destDataType = "CorbaPort";
					*/
					break;
				}
			}
		}
	}

	return result;
}
#endif


int linkComponent(std::string srcCompPort, std::string destCompPort)
{
	std::vector<std::string>	compNames;
	std::string					destCompName, srcCompName;
	std::string					destPort, srcPort;
	PortService					pSrcComp, pDestComp;


	// 接続先コンポーネント名をインスタンス名に変更する？？
	compNames = string_token(srcCompPort, " =:\r\n");
	srcCompName	= CREATE_RTC_NAME(compNames[0]);
	srcPort		= compNames[1];

	// 接続先コンポーネント名をインスタンス名に変更する？？
	compNames = string_token(destCompPort, " =:\r\n");
	destCompName	= CREATE_RTC_NAME(compNames[0]);
	destPort		= compNames[1];

	pSrcComp = findComponentPort(srcCompPort, out_port);
	pDestComp = findComponentPort(destCompPort, in_port);

	/*if((pSrcComp != NULL) && (pDestComp != NULL))*/{
		ReturnCode_t		ret;
		ConnectorProfile	prof;
		std::string			profName;

		profName = srcPort + "_" + destPort;
		prof.connector_id = "";
		prof.name = CORBA::string_dup(profName.c_str());
		prof.ports.length(2);
#ifdef RTM_Ver4
		prof.ports[0] = srcPorts[srcPortCnt];
		prof.ports[1] = destPorts[destPortCnt];
#else
		prof.ports[0] = (pSrcComp);
		prof.ports[1] = (pDestComp);
#endif

		/*if(srcDataType != "CorbaPort")*/{
#ifdef RTM_Ver4
			CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.interface_type", "CORBA_Any"));
			CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.dataflow_type", "Push"));
			CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.subscription_type", "Flush"));
#else
			/*
			 * Ver1.0 からは
			 * InterfaceType	= corba_cdr
			 * DatafllowType	= push	/ pull
			 * SubscriptionType	= flush	/ new	/ periodic
			 *
			 */
			CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.interface_type", "corba_cdr"));
			CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.dataflow_type", "push"));
			CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.subscription_type", "flush"));
#endif
		}
#ifdef RTM_Ver4
		ret = srcPorts[srcPortCnt]->connect(prof);
#else
		ret = pSrcComp->connect(prof);
#endif
	}

}
#endif

#if 0
/**
 * @brief RTCリンクを行う
 * @return リンク実行結果
 */
int linkComponent()
{
	std::vector<std::string>			compNames, connectList;
	std::vector<std::string>::iterator	it, compIt;

	connectList = iniFileMap[LINK_SECTION];
	for(it = connectList.begin(); it != connectList.end(); it++){
		CORBA::Object_ptr		pSrcComp, pDestComp;
		std::string				destCompName, srcCompName;
		std::string				destPort, srcPort;

		/*
		 * コンポーネント名：ポート名 = コンポーネント名：ポート名
		 * となっているので、1行で２つのコンポーネントを入手できる
		 */
		// " =:\r\n"でパース
		compNames = string_token(*it, " =:\r\n");

		srcCompName		= CREATE_RTC_NAME(compNames[0]);
		srcPort			= compNames[1];
		destCompName	= CREATE_RTC_NAME(compNames[2]);
		destPort			= compNames[3];

		try{
#if 0
			pSrcComp = findComponent(srcCompName.c_str());
			pDestComp = findComponent(destCompName.c_str());
#else
			RTC::CorbaNaming name_s(corbaORB, "localhost");
			std::cout << "comp comp comp = " << srcCompName.c_str() << std::endl;
			pSrcComp = name_s.resolve(srcCompName.c_str());
			pDestComp = name_s.resolve(destCompName.c_str());
#endif
			if((pDestComp != NULL) && (pSrcComp != NULL)){
				// コンポーネントが見つかった
				CorbaConsumer<RTC::RTObject>		destComp, srcComp;
#ifdef RTM_Ver4
				PortList_var		destPorts, srcPorts;
#else
				PortServiceList		*destPorts, *srcPorts;
#endif
				PortProfile			*portProfile;
				std::string			curPort, profName;
				int					destPortCnt, srcPortCnt, destPortMax, srcPortMax;
				std::string			srcDataType, destDataType, portType;
				SDOPackage::NVList	propListNV;
				RTC::Properties		portProp;
#if DUMP_PORT_DATA
				size_t	propCnt;
				std::vector<std::string>	propList;
#endif
				destComp.setObject(pDestComp);
				destPorts = destComp->get_ports();
				destPortMax = (int)(destPorts->length());

				srcComp.setObject(pSrcComp);
				srcPorts = srcComp->get_ports();
				srcPortMax = (int)(srcPorts->length());

				profName = srcPort + "_" + destPort;
				/* ポートの接続を行う */
				// 用のあるポートを探す
				for(destPortCnt = 0; destPortCnt < destPortMax; destPortCnt++){
#ifdef RTM_Ver4
					portProfile = destPorts[destPortCnt]->get_port_profile();
#else
					portProfile = ((*destPorts)[destPortCnt])->get_port_profile();
#endif

					/* ポートのプロパティを取得 */
					propListNV = portProfile->properties;
					portProp = NVUtil::toProperties(propListNV);
					portType = portProp["port.port_type"];
					if((portType != "DataInPort") && (portType != "CorbaPort"))	continue;

					// ポート名取得
					curPort = portProfile->name;
#if DUMP_PORT_DATA
cout << "確定ポート名 = " << destPort << endl;
cout << "探索中 ポートプロパティダンプ開始 (data_in)" << endl;
cout << "ポート名 = " << curPort << endl;
					/* ポートのプロパティを取得 */
					propListNV = portProfile->properties;
					portProp = NVUtil::toProperties(propListNV);
					propList = portProp.propertyNames();
					printf("相手ポートプロパティは %d あります\n", portProp.size());
					for(propCnt = 0; propCnt < propList.size(); propCnt++){
						std::cout << propList[propCnt];
						std::cout << " = ";
						std::cout << portProp[propList[propCnt]];
						std::cout << "\n";
					}
cout << "ポートプロパティダンプ終了" << endl;
#endif

					if(destPort == curPort){
						// ポートが見つかった
						printf("ポートが見つかりました [%d]\n", destPortCnt);
						if(portType != "CorbaPort")
							destDataType = portProp["dataport.data_type"];
						else
							destDataType = "CorbaPort";
						break;
					}

				}
				for(srcPortCnt = 0; srcPortCnt < srcPortMax; srcPortCnt++){
#ifdef RTM_Ver4
					portProfile = srcPorts[srcPortCnt]->get_port_profile();
#else
					portProfile = ((*srcPorts)[srcPortCnt])->get_port_profile();
#endif
					propListNV = portProfile->properties;
					portProp = NVUtil::toProperties(propListNV);
					portType = portProp["port.port_type"];
					if((portType != "DataOutPort") && (portType != "CorbaPort"))	continue;

					// ポート名取得
					curPort = portProfile->name;
#if DUMP_PORT_DATA
cout << "確定ポート名 = " << srcPort << endl;
cout << "ポートプロパティダンプ開始 (DataOutPort)" << endl;
cout << "ポート名 = " << curPort << endl;
					/* ポートのプロパティを取得してみる */
					propListNV = portProfile->properties;
					portProp = NVUtil::toProperties(propListNV);
					propList = portProp.propertyNames();
					printf("相手ポートプロパティは %d あります\n", portProp.size());
					for(propCnt = 0; propCnt < propList.size(); propCnt++){
						std::cout << propList[propCnt];
						std::cout << " = ";
						std::cout << portProp[propList[propCnt]];
						std::cout << "\n";
					}
cout << "ポートプロパティダンプ終了" << endl;
#endif

					if(srcPort == curPort){
						// ポートが見つかった
						printf("ポートが見つかりました [%d]\n", srcPortCnt);
						if(portType != "CorbaPort")
							srcDataType = portProp["dataport.data_type"];
						else
							srcDataType = "CorbaPort";
						break;
					}

				}
				if(srcDataType == destDataType){
					ReturnCode_t ret;
					ConnectorProfile prof;

					prof.connector_id = "";
					prof.name = CORBA::string_dup(profName.c_str());
					prof.ports.length(2);
#ifdef RTM_Ver4
					prof.ports[0] = srcPorts[srcPortCnt];
					prof.ports[1] = destPorts[destPortCnt];
#else
					prof.ports[0] = ((*srcPorts)[srcPortCnt]);
					prof.ports[1] = ((*destPorts)[destPortCnt]);
#endif
					if(srcDataType != "CorbaPort"){
#ifdef RTM_Ver4
						CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.interface_type", "CORBA_Any"));
						CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.dataflow_type", "Push"));
						CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.subscription_type", "Flush"));
#else
						/*
						 * Ver1.0 からは
						 * InterfaceType	= corba_cdr
						 * DatafllowType	= push	/ pull
						 * SubscriptionType	= flush	/ new	/ periodic
						 *
						 */
						CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.interface_type", "corba_cdr"));
						CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.dataflow_type", "push"));
						CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.subscription_type", "flush"));
#endif
					}
#ifdef RTM_Ver4
					ret = srcPorts[srcPortCnt]->connect(prof);
#else
					ret = ((*srcPorts)[srcPortCnt])->connect(prof);
#endif
				}
				else{
					printf("ポートの型が一致しません\n");
				}
			}
			else{
				printf("ポートが取得できなかった\n");
			}
		}
		catch(...){
		}

	}
	// 確定ではないが、リンクを張るのに時間がかかってる気がする
	sleep(1);
	return 0;
}
#endif
#if 1
/**
 * @brief コンポーネント群を終了する
 * @return 終了結果
 */
int exitComponent()
{
	int	resutl = 0;
#if 0
	// CORBA オブジェクトの取得
	std::vector< std::string >::iterator	it;
	std::string				lastComponent;

	for(it = compList.begin(); it != compList.end(); it++){
		// コンポーネント起動
		// 起動する前にCORBAサーバに登録されていいる事を確認する
		CORBA::Object_ptr pComp;
		try{
			std::string compName;

			compName = CREATE_RTC_NAME(*it);
			pComp = findComponent(compName.c_str());
			if(pComp != NULL){
				// コンポーネントが見つかった
				CorbaConsumer<RTC::RTObject>		compObj;
				ExecutionContextList_var			contList;

				compObj.setObject(pComp);
				compObj->exit();

				continue;
			}
		}
		catch(...){
			/*
			 * コンポーネントが登録されていない・・・
			 * 起動に失敗したらしい
			 */
		}
	}
#endif
	return resutl;
}
#endif
#if 0
/**
 * @brief コンポーネント群をアクティブ化する
 * @return アクティブ化結果
 */
int activateComponent(std::string compName, bool activate)
{
	int	result = 0;

	// CORBA オブジェクトの取得
	CORBA::Object_ptr pComp;
	try{
		pComp = findComponent(compName.c_str());
		if(pComp != NULL){
			ReturnCode_t ret;

			// コンポーネントが見つかった
			CorbaConsumer<RTC::RTObject>		compObj;
			ExecutionContextList_var			contList;

			compObj.setObject(pComp);

			// RTCコンテキストの取得
#ifdef RTM_Ver4
			contList = compObj->get_contexts();
#else
			contList = compObj->get_owned_contexts();
#endif
			// コンテキスト数の確認 (正常だと１になる)
			if(contList->length() == 1){
				if(activate)
					ret = contList[0]->activate_component(compObj._ptr());
				else
					ret = contList[0]->deactivate_component(compObj._ptr());
			}
		}
	}
	catch(...){
		/*
		 * コンポーネントが登録されていない・・・
		 * 起動に失敗したらしい
		 */
	}

	return result;
}
#endif
#if 0
/**
 * @brief コンポーネント群を非活性化する
 * @return 非活性化結果
 */
int deactivateComponent(std::string compName)
{
	return componentControl(compName, cont_type_deactivate_comp);
}
#endif

/**
 * @brief コマンドラインをコマンド名とパラメタに分解する
 * @return 分解された文字列のリスト
 */
std::vector<std::string> commandLine_token(const std::string line)
{
	std::string::size_type index;
	std::string	key_word;
	std::vector<std::string> ret, compName;

	index = line.find("=");
	if(index != std::string::npos){
		key_word = line.substr(0, index);
		compName = string_token(key_word, " ");
		ret.push_back(compName[0]);
		key_word = line.substr(index+1, line.length()-index);
		ret.push_back(key_word);
	}

	return ret;
}

void createCommandList()
{
	CommandData	command_data;

	command_data.alias			= false;
	command_data.description	= "		(h / ?) : ヘルプを表示";
	command_data.func			= showCommandsList;
//	command_data.showCandidate	= candidateVoid;
	commandList.insert(command_val("help", command_data));			// アプリケーションの使用方法表示
	command_data.alias			= true;
	commandList.insert(command_val("h", command_data));			// アプリケーションの使用方法表示
	commandList.insert(command_val("?", command_data));			// アプリケーションの使用方法表示

	command_data.alias			= false;
	command_data.description	= "		(ls)	: 起動されているRTCを列挙する";
	command_data.func			= listupComponents;
	command_data.showCandidate	= candidateNaming;
	commandList.insert(command_val("ls", command_data));			// 起動済みコンポーネントの一覧表示

	command_data.alias			= false;
	command_data.description	= "		(ln)	: コンポーネント間リンクを行う";
	command_data.showCandidate	= candidateComponentPort;
	command_data.func			= linkComponents;
	commandList.insert(command_val("link", command_data));			// コンポーネント・ポート間リンクの作成
	command_data.alias = true;
	commandList.insert(command_val("ln", command_data));			// 起動済みコンポーネントの一覧表示

	command_data.alias = false;
	command_data.description = "		(sh)	: コンポーネントの起動を行う";
	command_data.showCandidate	= candidateFile;
	command_data.func			= runComponent;
	commandList.insert(command_val("shell", command_data));
	command_data.alias = true;
	commandList.insert(command_val("sh", command_data));

	command_data.alias = false;
	command_data.description = "		(ul) : コンポーネント間リンクを解除する";
	command_data.showCandidate	= candidateComponentPort;
	command_data.func			= unlinkComponents;
	commandList.insert(command_val("unlink", command_data));		// コンポーネント・ポート間リンクの破棄
	command_data.alias = true;
	commandList.insert(command_val("ul", command_data));			// 起動済みコンポーネントの一覧表示

	command_data.alias			= false;
	command_data.description	= "			: コンフィグレーションの設定を行う";
	command_data.showCandidate	= candidateComponentConfig;
	command_data.func			= setComponentConfiguration;
	commandList.insert(command_val("setconfig", command_data));		// コンポーネントコンフィグレーションの変更

	command_data.alias			= false;
	command_data.description	= "			: コンフィグレーションの表示を行う";
	command_data.showCandidate	= candidateComponent;
	command_data.func			= showComponentConfiguration;
	commandList.insert(command_val("showconfig", command_data));	// コンポーネントコンフィグレーションの表示

	command_data.alias			= false;
	command_data.description	= "		(ls -naming) : 参照しているネーミングサーバを列挙する";
	command_data.showCandidate	= candidateVoid;
	command_data.func			= listupNamingServers;
	commandList.insert(command_val("lns", command_data));			// 参照済みネーミングサーバの一覧表示

	command_data.alias			= false;
	command_data.description	= "			: NamingServerの参照を行う";
	command_data.showCandidate	= candidateNaming;
	command_data.func			= referenceNamingServer;
	commandList.insert(command_val("ref", command_data));			// ネーミングサーバの参照追加

	command_data.alias			= false;
	command_data.description	= "	(a)	: アクティブ化を行う";
	command_data.showCandidate	= candidateComponent;
	command_data.func			= activate_component;
	commandList.insert(command_val("activate", command_data));		// コンポーネントのアクティベート
	command_data.alias			= true;
	commandList.insert(command_val("a", command_data));			// 起動済みコンポーネントの一覧表示

	command_data.alias			= false;
	command_data.description	= "	(d)	: 非アクティブ化を行う";
	command_data.func			= deactivate_component;
	command_data.showCandidate	= candidateComponent;
	commandList.insert(command_val("deactivate", command_data));	// コンポーネントのデアクティベート
	command_data.alias			= true;
	commandList.insert(command_val("d", command_data));			// 起動済みコンポーネントの一覧表示

	command_data.alias = false;
	command_data.description = "			: リセットを行う";
	command_data.showCandidate	= candidateComponent;
	command_data.func			= reset_component;
	commandList.insert(command_val("reset", command_data));			// コンポーネントのリセット

	command_data.alias			= false;
	command_data.description	= "			: コンポーネントのポート情報を表示する";
	command_data.func			= showComponentInfo;
	command_data.showCandidate	= candidateComponent;
	commandList.insert(command_val("show", command_data));			// ポート情報の表示
/*
	command_data.alias = false;
	command_data.description = "	(f)	: 終了処理を行う";
	command_data.showCandidate	= candidateComponent;
	command_data.func			= test;
	commandList.insert(command_val("finalize", command_data));		// コンポーネントの終了
	command_data.alias = true;
	commandList.insert(command_val("f", command_data));			// 起動済みコンポーネントの一覧表示
*/
	command_data.alias = false;
	command_data.description = "		(e)	: 終了させる";
	command_data.showCandidate	= candidateComponent;
	command_data.func			= exit_component;
	commandList.insert(command_val("exit", command_data));			// コンポーネントの終了
	command_data.alias = true;
	commandList.insert(command_val("e", command_data));			// 起動済みコンポーネントの一覧表示

	command_data.alias = false;
	command_data.description = "		(s)	: 動作を開始させる";
	command_data.showCandidate	= candidateComponent;
	command_data.func			= start_component;
	commandList.insert(command_val("start", command_data));			// コンポーネントの開始
	command_data.alias = true;
	commandList.insert(command_val("s", command_data));			// 起動済みコンポーネントの一覧表示

	command_data.alias = false;
	command_data.description = "		(x)	: 動作を停止させる";
	command_data.showCandidate	= candidateComponent;
	command_data.func			= stop_component;
	commandList.insert(command_val("stop", command_data));			// コンポーネントの停止
	command_data.alias = true;
	commandList.insert(command_val("x", command_data));			// 起動済みコンポーネントの一覧表示

	command_data.alias			= false;
	command_data.description	= "		(r)	: スクリプトファイルの実行を行う";
	command_data.func			= runScript;
	command_data.showCandidate	= candidateFile;
	commandList.insert(command_val("run", command_data));			// コンポーネント起動
	command_data.alias			= true;
	commandList.insert(command_val("r", command_data));

	command_data.alias			= false;
	command_data.description	= "			: 読み込みスクリプトを表示する";
	command_data.func			= showScriptList;
	command_data.showCandidate	= candidateVoid;
	commandList.insert(command_val("list", command_data));			// 起動済みコンポーネントの一覧表示

	command_data.alias			= false;
	command_data.description	= "			: コマンド履歴を表示する";
	command_data.func			= showHistoryList;
	command_data.showCandidate	= candidateVoid;
	commandList.insert(command_val("history", command_data));			// 起動済みコンポーネントの一覧表示

	command_data.alias			= false;
	command_data.description	= "			: 情報ファイルの読み込みを行う";
	command_data.func			= loadScriptFile;
	command_data.showCandidate	= candidateFile;
	commandList.insert(command_val("load", command_data));			// 情報ファイルの読み込み

	command_data.alias			= false;
	command_data.description	= "			: 情報ファイルの作成を行う";
	command_data.func			= saveScriptFile;
	command_data.showCandidate	= candidateFile;
	commandList.insert(command_val("save", command_data));			// 情報ファイルの書き込み

	command_data.alias			= false;
	command_data.description	= "			: 現在の接続状態を取得する";
	command_data.func			= getCurrentLinkInfo;
	command_data.showCandidate	= candidateComponent;
	commandList.insert(command_val("showlink", command_data));

	command_data.alias = false;
	command_data.description = "			: コンポーネントファイルの存在確認を行う";
	command_data.showCandidate	= candidateFile;
	command_data.func			= test;
	commandList.insert(command_val("exist", command_data));			// 実行モジュール存在確認

	command_data.alias			= false;
	command_data.description	= "		(q) : アプリケーションを終了します";
	command_data.func			= quitApplication;
	command_data.showCandidate	= candidateVoid;
	commandList.insert(command_val("quit", command_data));
	command_data.alias			= true;
	commandList.insert(command_val("q", command_data));

	command_data.alias			= false;
	command_data.description	= "		(test) : テスト";
	command_data.func			= test;
	command_data.showCandidate	= candidateVoid;
	commandList.insert(command_val("test", command_data));

}

int test(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	cout << "\t\t===== 未実装 =====" << endl;

#if 0
	// TerminalServer0
	CORBA::Object_ptr	pTarget;

	pTarget = findComponent("TerminalServer0");
//	pTarget = corbaORB->resolve_initial_references("TerminalServer0");
//	pTarget = corbaORB->resolve_initial_references("InterfaceRepository");
//	pTarget = corbaORB->resolve_initial_references("localhost");
	if(CORBA::is_nil(pTarget) == 0){
//		CORBA::InterfaceDef_ptr	*corbaIf;
//		CORBA::_objref_InterfaceDef	*corbaIf;
		CORBA::InterfaceDef_var	corbaIf;

cout << "OK 1?" << endl;
		corbaIf = pTarget->_get_interface();
cout << "OK 1-E?" << endl;
		if(CORBA::is_nil(corbaIf) == 0){
			CORBA::InterfaceDefSeq	*ifList;
			CORBA::InterfaceDef::FullInterfaceDescription_var	fullInfo;
#if 0
cout << "OK 2?" << endl;
			ifList = (corbaIf)->base_interfaces();
cout << "OK 2-E?" << endl;
#if 1
			for(unsigned int n = 0; n < ifList->length(); n++){
cout << "OK 3?" << endl;
				fullInfo = ((*ifList)[n])->describe_interface();
				cout << "IF = " << fullInfo->name << endl;
			}
#endif
#else
			fullInfo = corbaIf->describe_interface();
			for(unsigned i = 0; i < fullInfo->operations.length(); i++){
				cout << fullInfo->operations[i].name << endl;
			}
#endif
		}
		else{
			cout << "interface not found!" << endl;
		}
	}
	else{
		cout << "Target not found!" << endl;
	}
	cout << "END end END" << endl;
#else
	try {
//		CORBA::String_var idlName = "/home/rtc-center/build_temp/OpenRTM-aist-1.0.0/src/lib/rtm/idl/RTC.idl";
		CORBA::String_var idlName = "TerminalServer0";

//		CORBA::ORB_ptr orb = /*CORBA::ORB_init(argc, argv)*/ corbaORB;
		CORBA::ORB::ObjectIdList *serviceList = corbaORB->list_initial_services();
		if(serviceList != NULL){
			for(unsigned int j = 0; j < serviceList->length(); j++){
				cout << ((*serviceList)[j]) << endl;
			}
			cout << "\n\n\n" << endl;
		}
#if 1
		CORBA::Object_ptr p_if = corbaORB->resolve_initial_references("InterfaceRepository");
		if(CORBA::is_nil(p_if) == 0)		cout << "pif" << endl;
		else								cout << "aaa" << endl;
		CORBA::Repository_ptr rep = CORBA::Repository::_narrow( p_if );
		if(CORBA::is_nil(rep) == 0)			cout << "rep" << endl;
		else								cout << "vvv" << endl;
		CORBA::Contained_var contained = rep->lookup("::ServiceChange");
//		CORBA::Contained_var contained = rep->lookup("::PA10Ctrl");
		if(CORBA::is_nil(contained) == 0)	cout << "contained" << endl;
		else								cout << "ddd" << endl;
		CORBA::InterfaceDef_var intDef = CORBA::InterfaceDef::_narrow(contained);
		if(CORBA::is_nil(intDef) == 0)		cout << "intDef" << endl;
		else								cout << "www" << endl;

		/*if (intDef != CORBA::InterfaceDef::_nil()) */{
			CORBA::InterfaceDef::FullInterfaceDescription_var fullDesc = intDef->describe_interface();
			cout << "Operations:" << endl;
			for(CORBA::ULong i = 0; i < fullDesc->operations.length(); i++){
				cout << "\t" << fullDesc->operations[i].name << endl;
				for(CORBA::ULong j = 0; j < fullDesc->operations[i].parameters.length(); j++){
					cout << "\t\t";
					cout << fullDesc->operations[i].parameters[j].name;
					cout << " / ";
					cout << fullDesc->operations[i].parameters[j].type;
					cout << " / ";
					cout << fullDesc->operations[i].parameters[j].type_def;
					cout << " / ";
					cout << fullDesc->operations[i].parameters[j].mode << endl;
				}
			}

			cout << "Attributes:" << endl;
			for(unsigned int i = 0; i < fullDesc->attributes.length(); i++){
				cout << "\t" << fullDesc->attributes[i].name << endl;
			}
		}
/*
		else
	         cout << "idlName is not an interface: " << idlName << endl;
*/
#endif
	} catch (const CORBA::Exception& excep) {
		cerr << "Exception occurred ..." << endl;
//		cerr << excep << endl;
//		exit(1);
	}
#endif
	return 1;
}


int runComponent(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	// コマンド コンポーネント名 パラメタ ・・・
	pid_t	cpid;
	int		pipe_fd[2];
	std::string	compFile, cpidStr;

	if(cmnd_list->size() < 2) return 1;

	compFile = (*cmnd_list)[1];

	if (pipe(pipe_fd) == -1) {
		// パイプ作成エラー
		cout << "起動失敗" << endl;
		return 1;
	}

	cpid = fork();
	if(cpid == -1){
		cout << "自分の起動に失敗した" << endl;
		return 1;
	}
	cpidStr = "" + cpid;
	compProcessList.insert(comp_val(cpidStr, compFile));
	if(cpid == 0){
		int				cnt, sys_ret;
		ssize_t	readSize = 1, totalSize = 0;
		size_t	commandLineLen = 0;
		char	*lpszCommandLine = 0;

		// 子プロセスが作動した
		// パイプ読み込み待機
		// パイプから取得した文字列でプロセスを起動する
		close(pipe_fd[1]);
		cnt = 0;
		while(readSize > 0){
			commandLineLen = DEF_BUFF_SIZE * (cnt + 1);
			lpszCommandLine = (char*)realloc(lpszCommandLine, commandLineLen);
			readSize = read(pipe_fd[0], &(lpszCommandLine[totalSize]), commandLineLen);
			totalSize = totalSize + readSize;
			cnt++;
		}
		close(pipe_fd[0]);

		// コマンドライン文字列をパースして、コマンド名とパラメタに分解する
//cout << "CommandLine = " << lpszCommandLine << endl;
		{
			std::string	strCommandLine(lpszCommandLine);
			free(lpszCommandLine);

			sys_ret = system(strCommandLine.c_str());
			exit(EXIT_SUCCESS);
		}
	}
	else{
		size_t		writeLen;
		std::string	commandLine = "";
		char	*lpszCommandLine = 0;

		// 継続処理・・・パイプに起動するコンポーネントとパラメタを流す
		//commandLine = command_str;
		std::vector<std::string>::iterator it = cmnd_list->begin();
		it++;
		for(; it != cmnd_list->end(); it++){
			commandLine = commandLine + *it + " ";
		}
//		commandLine = commandlineMap[commandLine];
//		if(commandLine.length() == 0){
//			commandLine = *it;
//		}
//cout << "WriteCommand = " << commandLine << endl;
		lpszCommandLine = (char*)realloc(lpszCommandLine, commandLine.length() + 2);
		memset(lpszCommandLine, 0, commandLine.length() + 2);
		strcpy(lpszCommandLine,commandLine.c_str());

		close(pipe_fd[0]);
		writeLen = write(pipe_fd[1], lpszCommandLine, commandLine.length() + 2);

		close(pipe_fd[1]);
	}

	return 1;
}

int unlinkComponents(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	std::vector< std::string >				compInfo;
	std::vector< std::string >::iterator	it;
	std::string	srcComp, destComp, srcPort, destPort;
	std::string name, if_type = "corba_cdr", df_type = "push", ss_type = "flush";
	std::string temp;
	RTC::PortService_ptr	srcInfo;
	ReturnCode_t		ret;

	//if(cmnd_list->size() < 3)	return 1;
	compInfo = string_token((*cmnd_list)[1], "@");
	srcComp = compInfo[0];
	srcPort = compInfo[1];
	// 接続元ポート情報取得
	srcInfo = getPortInfo(srcComp, srcPort);

	ret = srcInfo->disconnect_all();
	/*
	 * ret = srcInfo->disconnect(connect_name);
	 */
	// おまじない！
	sleep(2);
	if(ret == RTC_OK)
		cout << "すべて解除できたと思います。" << endl;
	else
		cout << "失敗したと思います。" << endl;

	return 1;
}
int linkComponents(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	std::vector< std::string >				compInfo;
	std::vector< std::string >::iterator	it;
	std::string	srcComp, destComp, srcPort, destPort;
	std::string name, if_type = "corba_cdr", df_type = "push", ss_type = "flush";
	std::string temp;

	if(cmnd_list->size() < 3)	return 1;

	// 接続元コンポーネント名・ポート名を取得
#if 0
	compInfo = string_token((*cmnd_list)[1], "/");
	temp = "";
	for(it = compInfo.begin(); it != compInfo.end(); it++){
		srcComp = temp;
		temp = temp + "/" + *it;
		srcPort = *it;
	}
#else
	compInfo = string_token((*cmnd_list)[1], "@");
	srcComp = compInfo[0];
	srcPort = compInfo[1];
	//cout << "Comp = " << srcComp << " / Port = " << srcPort << endl;
#endif

	// 接続先コンポーネント名・ポート名を取得
#if 0
	compInfo = string_token((*cmnd_list)[2], "/");
	temp = "";
	for(it = compInfo.begin(); it != compInfo.end(); it++){
		destComp = temp;
		temp = temp + "/" + *it;
		destPort = *it;
	}
#else
	compInfo = string_token((*cmnd_list)[2], "@");
	destComp = compInfo[0];
	destPort = compInfo[1];
	//cout << "Comp = " << destComp << " / Port = " << destPort << endl;
#endif

	name = srcPort + "_" + destPort;

	// パラメタを取得
	for(unsigned int cnt = 3; cnt < cmnd_list->size(); cnt++){
		compInfo = string_token((*cmnd_list)[cnt], " =");
		if(compInfo.size() == 2){
			if(compInfo[0].compare("name") == 0)	name	= compInfo[1];
			if(compInfo[0].compare("if") == 0)		if_type	= compInfo[1];
			if(compInfo[0].compare("df") == 0)		df_type	= compInfo[1];
			if(compInfo[0].compare("ss") == 0)		ss_type	= compInfo[1];
		}
	}

	return linkComponent(srcComp, srcPort, destComp, destPort, name, if_type, df_type, ss_type);
}

int getCurrentLinkInfo(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	// showlinkinfo NamingServer or Component
	// 現在の接続状態をすべて取得する
	// 出力ポートを主役として、左側は常に出力ポート
	// サービスポートは・・・入出力判別ができないかも・・・頑張ってプロパティを見れば判別できそうだが当面はやらない方向で

	// まずは見渡す限りのすべてのコンポーネントを列挙する？
	enumNamingServers(createLinkInfoCallback, CALLBACK_TYPE_COMP);

	return 1;
}

void createLinkInfoCallback(std::string name, std::string kind, int type, int layered, std::string path, CORBA::Object_ptr pTarget)
{
	if(type & CALLBACK_TYPE_COMP){
		// ポート情報を取得する
		CorbaConsumer<RTC::RTObject>	targetComp;
		PortServiceList					*pTargetPorts;
		PortProfile						*portProfile;
		int								portCnt, portMax;

		SDOPackage::NVList	propListNV;
		RTC::Properties		portProp;
		std::string			portType;

		ConnectorProfileList	connectList;
		ComponentProfile		*compProfile;

		if(kind.length() == 0)	return;

		targetComp.setObject(pTarget);

		compProfile = targetComp->get_component_profile();
		pTargetPorts = targetComp->get_ports();
		portMax = (int)(pTargetPorts->length());
		for(portCnt = 0; portCnt < portMax; portCnt++){
			portProfile = ((*pTargetPorts)[portCnt])->get_port_profile();
			propListNV = portProfile->properties;
			portProp = NVUtil::toProperties(propListNV);
			if(strcmp(portProp["port.port_type"].c_str(), "DataInPort") == 0)
				continue;

#if 1
			connectList = portProfile->connector_profiles;
			for(unsigned int i = 0; i < connectList.length(); i++){
				for(unsigned int j = 0; j < connectList[i].ports.length(); j++){
					if(strcmp((char*)(portProfile->name), (char*)(connectList[i].ports[j]->get_port_profile()->name)) != 0){
						std::string		targetPath;

						targetPath = getRegisterNamingName(connectList[i].ports[j]->get_port_profile()->owner);

						// コマンド名出力
						cout << "link ";
						// コンポーネントパス出力
						cout << path << "/";
						// コンポーネント名出力
						cout << name << "@";
						// ポート名出力
						std::cout << portProfile->name << " ";

						// 接続先コンポーネント名出力
						cout << targetPath << "@";
						// 接続先ポート名出力
						cout << connectList[i].ports[j]->get_port_profile()->name << endl;
					}
				}
			}
#else
			std::cout << "\t\tプロパティ : " << std::endl;
			propListNV = portProfile->properties;
			portProp = NVUtil::toProperties(propListNV);
			for(unsigned int n = 0 ; n < propListNV.length(); n++){
				std::cout << "\t\t\t" << propListNV[n].name << " : " << portProp[(char *)(propListNV[n].name)] << std::endl;
			}

			std::cout << "\t\t接続情報 : " << std::endl;
			connectList = portProfile->connector_profiles;
			portProp = NVUtil::toProperties(propListNV);
			for(unsigned int i = 0; i < connectList.length(); i++){
				std::cout << "\t\t\t接続名 : " << connectList[i].name << std::endl;
				std::cout << "\t\t\tプロパティ : " << std::endl;
				propListNV = connectList[i].properties;
				for(unsigned int n = 0 ; n < propListNV.length(); n++){
					std::cout << "\t\t\t\t" << propListNV[n].name << " : " << portProp[(char*)(propListNV[n].name)] << std::endl;
				}

				std::cout << "\t\t\t接続先ポート名 : " << std::endl;
				for(unsigned int j = 0; j < connectList[i].ports.length(); j++){
					if(strcmp((char*)(portProfile->name), (char*)(connectList[i].ports[j]->get_port_profile()->name)) != 0){
						std::string		targetPath;

						targetPath = getRegisterNamingName(connectList[i].ports[j]->get_port_profile()->owner);
						std::cout << "\t\t\t\t" << targetPath << std::endl;

						// ポート名表示
						std::cout << "\t\t\t\t\t" << connectList[i].ports[j]->get_port_profile()->name << std::endl;
					}
				}
			}
#endif
		}
	}
}


int deactivate_component(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	int	result;

	for(unsigned int cnt = 1; cnt < cmnd_list->size(); cnt++){
		//result = deactivateComponent((*cmnd_list)[cnt]);
		result = componentControl((*cmnd_list)[cnt], cont_type_deactivate_comp);
	}
	return 1;
}

int exit_component(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	int	result;

	for(unsigned int cnt = 1; cnt < cmnd_list->size(); cnt++){
		//result = deactivateComponent((*cmnd_list)[cnt]);
		result = componentControl((*cmnd_list)[cnt], cont_type_exit_comp);
	}
	return 1;
}

int activate_component(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	int	result;

	for(unsigned int cnt = 1; cnt < cmnd_list->size(); cnt++){
		//result = activateComponent((*cmnd_list)[cnt]);
		if(((*cmnd_list)[cnt]).compare("-all") != 0){
			result = componentControl((*cmnd_list)[cnt], cont_type_activate_comp);
		}
		else{
			// 全コンポーネントをアクティブ化する
			enumNamingServers(activateComponentCallback, CALLBACK_TYPE_COMP);
		}
	}
	return 1;
}

void activateComponentCallback(std::string name, std::string kind, int type, int layered, std::string path, CORBA::Object_ptr pComp)
{
	int	result;

	result = componentControl(name, cont_type_activate_comp, pComp);

	return;
}

int stop_component(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	int	result;

	for(unsigned int cnt = 1; cnt < cmnd_list->size(); cnt++){
		//result = activateComponent((*cmnd_list)[cnt]);
		result = componentControl((*cmnd_list)[cnt], cont_type_start_comp);
	}
	return 1;
}
int start_component(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	int	result;

	for(unsigned int cnt = 1; cnt < cmnd_list->size(); cnt++){
		//result = activateComponent((*cmnd_list)[cnt]);
		result = componentControl((*cmnd_list)[cnt], cont_type_stop_comp);
	}
	return 1;
}
int reset_component(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	int	result;

	for(unsigned int cnt = 1; cnt < cmnd_list->size(); cnt++){
		//result = activateComponent((*cmnd_list)[cnt]);
		result = componentControl((*cmnd_list)[cnt], cont_type_reset_comp);
	}
	return 1;
}

int runScript(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	if(cmnd_list->size() == 1){
		for(std::vector<std::string>::iterator it = commandDoc.begin(); it != commandDoc.end(); it++){
			cout << *it << endl;
			run_command(*it);
		}
	}
	else{
		int		ret;
		std::vector<std::string>	param;
		// ファイル指定になっている
		param.push_back("load");
		param.push_back("-run");
		param.push_back((*cmnd_list)[1]);

		ret = loadScriptFile(command_str, &param);
	}

	return 1;
}

int showScriptList(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	for(std::vector<std::string>::iterator it = commandDoc.begin(); it != commandDoc.end(); it++){
		std::cout << *it << std::endl;
		fflush(stdout);
	}

	return 1;
}

int showHistoryList(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	for(std::vector<std::string>::iterator it = commandHistory.begin(); it != commandHistory.end(); it++){
		std::cout << *it << std::endl;
		fflush(stdout);
	}

	return 1;
}

int loadScriptFile(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	int				ret_code;
	std::string		fileName = "";
	bool			runScriptFlg = false;

	for(unsigned int compCnt = 1; compCnt < (*cmnd_list).size(); compCnt++){
		if(((*cmnd_list)[compCnt]).compare("-run") == 0){
			runScriptFlg = true;
			continue;
		}
		else{
			fileName = ((*cmnd_list)[compCnt]);
		}
	}

	if(fileName.length() == 0){
		// ファイル名が指定されていない場合は「default.rbdf」を使用する。
		fileName = "default.rbdf";
	}

	// ファイルがすでにあるか確認する必要がある
	if(access(fileName.c_str(), F_OK) == 0){
		// ファイルがあるので読み込む
		std::string	cmdLine;
		char	buff[1024];
		char	*temp;
		FILE	*fd;

		commandDoc.clear();
		fd = fopen(fileName.c_str(), "r");
		do{
			// 1行読み込み
			memset(buff, 0, sizeof(buff));
			temp = fgets(buff, sizeof(buff), fd);
			if(temp != NULL){
				if(buff[strlen(buff) - 1] == 0x0a){
					buff[strlen(buff) - 1] = 0x00;
				}
				cmdLine = buff;
				if(cmdLine.length() > 0)
					commandDoc.push_back(cmdLine);
			}
		}while(temp != NULL);

		fclose(fd);

		if(runScriptFlg){
			std::vector<std::string>	cmdList;

			cmdList.push_back("");
			ret_code = runScript("", &cmdList);
		}
	}
	else{
		// ファイルが無い
		std::cout << "ファイル " << fileName << " は存在していません。"  << std::endl;
	}

	return 1;
}

int saveScriptFile(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	bool			bWrite = true;
	std::string		fileName, reply;

	if(cmnd_list->size() == 1){
		// ファイル名が指定されていない場合は「default.rbdf」を使用する。
		fileName = "default.rbdf";
	}
	else{
		fileName = (*cmnd_list)[1];
	}

	// ファイルがすでにあるか確認する必要がある
	if(access(fileName.c_str(), F_OK) == 0){
		std::cout << "ファイル " << fileName << " は既にあります。\n上書きしますか？ (yes / no)\n"  << std::endl;
		while(1){
			std::cout << "[yes (y)] / no (n) >> ";
			std::cin >> reply;
			if((reply.length() == 0) || (reply.compare("yes") == 0) || (reply.compare("y") == 0)){
				break;
			}
			if((reply.compare("n") == 0) || (reply.compare("no") == 0)){
				bWrite = false;
				break;
			}
		}
	}

	if(bWrite){
		// 書き込みを行う
		FILE	*fd;

		fd = fopen(fileName.c_str(), "w");
		//std::vector<std::string>	commandHistory
		for(std::vector<std::string>::iterator it = commandHistory.begin(); it != commandHistory.end(); it++){
			fprintf(fd, "%s\n", it->c_str());
		}
		fclose(fd);
	}

	return 1;
}

int setComponentConfiguration(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	CORBA::Object_ptr pTarget;

	if(cmnd_list->size() > 2){
		pTarget = findComponent((*cmnd_list)[1]);
		if(pTarget){
			std::string						curConfName;
			CorbaConsumer<RTC::RTObject>	targetComp;
			SDOPackage::Configuration_ptr	config;
			SDOPackage::NVList				propListNV;
			RTC::Properties					confProp;
//			SDOPackage::ConfigurationSetList	*configList;
			SDOPackage::ConfigurationSet	*configValue, *newValue;
			std::string					new_id, old_id;

			// コンポーネント構築
			targetComp.setObject(pTarget);
			config		= targetComp->get_configuration();
			configValue	= config->get_active_configuration_set();
			propListNV	= configValue->configuration_data;

			// 現在のコンフィグ名を退避
			old_id = configValue->id;
			// 新規コンフィグレーションを追加する
			newValue = configValue;

			// 追加コンフィグ名を生成
			new_id = (*cmnd_list)[1] + "_temp";
			newValue->id = new_id.c_str();

			// パラメタ数分回る
			for(unsigned int paramCnt = 2; paramCnt < cmnd_list->size(); paramCnt++){
				// パラメタをパラメタ名・値に分解する
				std::vector<std::string>	paramVal;

				paramVal = string_token((*cmnd_list)[paramCnt], " =");
				if(paramVal.size() == 2){
					// 分解完了したので、コンフィグレーションから該当するパラメタを探す
					for(unsigned int n = 0 ; n < propListNV.length(); n++){
						if(strcmp((char *)(propListNV[n].name), paramVal[0].c_str()) == 0){
							CORBA::Any		confVal;

							// 該当コンフィグパラメタ検出
							// 変数の型を確認し、設定値を変換する
							switch(propListNV[n].value.type()->kind()){		// CORBA::TypeCode
								case CORBA::tk_short:
									confVal <<= (short)atoi(paramVal[1].c_str());
									break;
								case CORBA::tk_long:
									confVal <<= (long)atoi(paramVal[1].c_str());
									break;
								case CORBA::tk_ushort:
									confVal <<= (unsigned short)atoi(paramVal[1].c_str());
									break;
								case CORBA::tk_ulong:
									confVal <<= (unsigned long)atoi(paramVal[1].c_str());
									break;
								case CORBA::tk_float:
									confVal <<= (float)atof(paramVal[1].c_str());
									break;
								case CORBA::tk_double:
									confVal <<= (double)atof(paramVal[1].c_str());
									break;
								case CORBA::tk_boolean:
									break;
								case CORBA::tk_char:
									break;
								case CORBA::tk_string:
									confVal <<= paramVal[1].c_str();
									break;
								case CORBA::tk_ulonglong:
								case CORBA::tk_longlong:
								case CORBA::tk_octet:
								case CORBA::tk_null:
								case CORBA::tk_void:
								case CORBA::tk_any:
								case CORBA::tk_TypeCode:
								case CORBA::tk_Principal:
								case CORBA::tk_fixed:
								case CORBA::tk_objref:
								case CORBA::tk_struct:
								case CORBA::tk_union:
								case CORBA::tk_enum:
								case CORBA::tk_sequence:
								case CORBA::tk_array:
								default:
									break;
							}


							for(unsigned int i = 0 ; i < newValue->configuration_data.length(); i++){
								if(strcmp((char *)(newValue->configuration_data[i].name), paramVal[0].c_str()) == 0){
									newValue->configuration_data[i].value = confVal;
								}
							}
						}
					}
				}
			}

			// 一度追加する
			if(config->add_configuration_set(*newValue))
				cout << "Add OK" << endl;
			else
				cout << "Add NG" << endl;

			// アクティブを変更
			if(config->activate_configuration_set(new_id.c_str()))
				cout << "変更2完了" << endl;
			else
				cout << "変更2失敗" << endl;

			// 旧データを変更する
			configValue->id = old_id.c_str();
			if(config->set_configuration_set_values(*configValue))
				cout << "変更2完了" << endl;
			else
				cout << "変更2失敗" << endl;

			// アクティブを変更
			if(config->activate_configuration_set(old_id.c_str()))
				cout << "変更2完了" << endl;
			else
				cout << "変更2失敗" << endl;

#if 0	//将来的にはコンフィグを追加しないで処理したいが、現在は追加しちゃう
			// 追加したコンフィグを削除する
			if(config->remove_configuration_set(new_id.c_str()))
				cout << "Del OK" << endl;
			else
				cout << "Del NG (" << newValue->id << endl;
#endif
		}
	}
	return 1;
}

int showComponentConfiguration(std::string command_str, std::vector<std::string>  *cmnd_list)
{
	CORBA::Object_ptr pTarget;

	if(cmnd_list->size() > 1){
		pTarget = findComponent((*cmnd_list)[1]);
		if(pTarget){
			std::string						curConfName;
			CorbaConsumer<RTC::RTObject>	targetComp;
			SDOPackage::Configuration_ptr	config;
			SDOPackage::NVList				propListNV;
			RTC::Properties					confProp;
			SDOPackage::ConfigurationSetList	*configList;
			SDOPackage::ConfigurationSet	*configValue;

			targetComp.setObject(pTarget);
			config = targetComp->get_configuration();
			configValue = config->get_active_configuration_set();
			propListNV = configValue->configuration_data;
#if 1
			confProp = NVUtil::toProperties(propListNV);

			curConfName = (char *)(configValue->id);
			std::cout << "\n現在のコンフィグレーション\t(" << configValue->id << " / " << configValue->description << ")" << std::endl;
			for(unsigned int n = 0 ; n < propListNV.length(); n++){
				std::cout << "\t" << propListNV[n].name << "\t: " << confProp[(char*)(propListNV[n].name)] << std::endl;
			}
			std::cout << "\n\tその他のコンフィグレーション" << std::endl;
			configList = config->get_configuration_sets();
			for(unsigned int i = 0; i < configList->length(); i++){
				if((strcmp((char *)((*configList)[i].id), "__widget__") != 0) && (strcmp((char *)((*configList)[i].id), curConfName.c_str()) != 0)){
					std::cout << "\t\tコンフィグレーション名 : " << (*configList)[i].id << " / " << (*configList)[i].description << std::endl;
					propListNV = (*configList)[i].configuration_data;
					confProp = NVUtil::toProperties(propListNV);
					for(unsigned int n = 0 ; n < propListNV.length(); n++){
						std::cout << "\t\t\t" << propListNV[n].name << "\t: " << confProp[(char*)(propListNV[n].name)] << std::endl;
					}
				}
			}
#endif
		}
	}
	return 1;
}


int showComponentInfo(std::string command_str, std::vector<std::string> *cmnd_list)
{
	bool	fullInfo = false;
	size_t compCnt = 0;
	CORBA::Object_ptr pTarget;

	for(compCnt = 1; compCnt < (*cmnd_list).size(); compCnt++){
		if(((*cmnd_list)[compCnt]).compare("-all") == 0){
			fullInfo = true;
			continue;
		}
		if(((*cmnd_list)[compCnt]).compare("-short") == 0){
			fullInfo = false;
			continue;
		}

		pTarget = findComponent((*cmnd_list)[compCnt]);
		if(pTarget){
			// ポート情報を取得する
			CorbaConsumer<RTC::RTObject>	targetComp;
			PortServiceList					*pTargetPorts;
			PortProfile						*portProfile;
			int								portCnt, portMax;

			SDOPackage::NVList	propListNV;
			RTC::Properties		portProp;
			std::string			portType;

			ConnectorProfileList	connectList;
			ComponentProfile		*compProfile;

			std::cout << "\n" << std::endl;
			targetComp.setObject(pTarget);

			// インスタンス名も出力しておく
			compProfile = targetComp->get_component_profile();
			std::cout << "インスタンス名\t: " << compProfile->instance_name << std::endl;
#if 1
			{
				SDOPackage::NVList	*pListNV;
				pListNV = targetComp->get_status_list();
				portProp = NVUtil::toProperties(*pListNV);
				if((*pListNV).length() == 0){
					cout << "なし" << endl;
				}
				else{
					cout << "---- get_status_list() ----" << endl;
					for(unsigned int n = 0 ; n < (*pListNV).length(); n++){
						std::cout << "\t" << (*pListNV)[n].name << " : " << portProp[(char *)((*pListNV)[n].name)] << std::endl;
					}
					cout << "---------------------------" << endl;
				}
			}

			//cout << "---- get_sdo_id() ----" << endl;
			{
				std::cout << "SDO ID\t: " << targetComp->get_sdo_id() << std::endl;
				//cout << "---------------------------" << endl;
			}

			//cout << "---- get_sdo_type() ----" << endl;
			{
				std::cout << "SDO Type\t: " << targetComp->get_sdo_type() << std::endl;
				//cout << "---------------------------" << endl;
			}

			{
				SDOPackage::ServiceProfileList	*sProfileList;

				sProfileList = targetComp->get_service_profiles();
				if(sProfileList->length() == 0){
					cout << "なし" << endl;
				}
				else{
					cout << "---- get_service_profiles() ----" << endl;
					for(int j = 0; sProfileList->length(); j++){
						std::cout << "ID = " << (*sProfileList)[j].id << std::endl;
						std::cout << "interface_type = " << (*sProfileList)[j].interface_type << std::endl;
						propListNV = (*sProfileList)[j].properties;
						portProp = NVUtil::toProperties(propListNV);
						for(unsigned int n = 0 ; n < propListNV.length(); n++){
							std::cout << "\t" << propListNV[n].name << " : " << portProp[(char *)(propListNV[n].name)] << std::endl;
						}
					}
					cout << "---------------------------" << endl;
				}
			}

			{
				SDOPackage::DeviceProfile 	*deviceProfile;

				deviceProfile = targetComp->get_device_profile();
				if(deviceProfile != NULL){
					cout << "---- get_device_profile() ----" << endl;
					std::cout << "Device Thype\t:" << deviceProfile->device_type << endl;
					std::cout << "Manufacturer\t:" << deviceProfile->manufacturer << endl;
					std::cout << "Model\t:" << deviceProfile->model << endl;
					std::cout << "Version\t:" << deviceProfile->version << endl;
//					NVList 	properties
					cout << "---------------------------" << endl;
				}
			}
/*
			cout << "---- get_service_profile() ----" << endl;
			{
				ServiceProfile 	serviceProfile;
				serviceProfile = get_service_profile (in UniqueIdentifier id) raises (InvalidParameter, NotAvailable, InternalError)
				cout << "---------------------------" << endl;
			}
*/
/*
			cout << "---- get_sdo_service() ----" << endl;
			{
				SDOService sdoService;
				sdoService = get_sdo_service (in UniqueIdentifier id) raises (InvalidParameter, NotAvailable, InternalError)
				cout << "---------------------------" << endl;
			}
*/
/*
			cout << "---- get_monitoring() ----" << endl;
			{
				Monitoring monitoring;

				monitoring = get_monitoring () raises (InterfaceNotImplemented, NotAvailable, InternalError)
				cout << "---------------------------" << endl;
			}
*/
			{
				SDOPackage::OrganizationList *organizationList;

				organizationList = targetComp->get_organizations();
				if(organizationList->length() == 0){
					cout << "なし" << endl;
				}
				else{
					cout << "---- get_organizations() ----" << endl;
					for(int j = 0; organizationList->length(); j++){
						std::cout << "ID = " << ((*organizationList)[j])->get_organization_id() << std::endl;
						/*
						std::cout << "interface_type = " << (*sProfileList)[j].interface_type << std::endl;
						propListNV = (*organizationList)[j].properties;
						portProp = NVUtil::toProperties(propListNV);
						for(unsigned int n = 0 ; n < propListNV.length(); n++){
							std::cout << "\t" << propListNV[n].name << " : " << portProp[(char *)(propListNV[n].name)] << std::endl;
						}
						*/
					}
					cout << "---------------------------" << endl;
				}
			}
#endif
			if(fullInfo){
				CorbaConsumer<RTC::RTObject>	pParentComp;

				std::cout << "型名\t\t: " << compProfile->type_name << std::endl;
				std::cout << "説明\t\t: " << compProfile->description << std::endl;
				std::cout << "バージョン\t: " << compProfile->version << std::endl;
				std::cout << "ベンダー\t: " << compProfile->vendor << std::endl;
				std::cout << "カテゴリー\t: " << compProfile->category << std::endl;
				std::cout << "プロパティ" << std::endl;
				//
				propListNV = compProfile->properties;
				portProp = NVUtil::toProperties(propListNV);
				if(propListNV.length() == 0){
					cout << "なし" << endl;
				}
				else{
					for(unsigned int n = 0 ; n < propListNV.length(); n++){
						std::cout << "\t" << propListNV[n].name << " : " << portProp[(char *)(propListNV[n].name)] << std::endl;
					}
					std::cout << "----" << std::endl;
				}

			}

			// コンポーネントの状態を取得する
			{
				RTC::ExecutionContextList_var			contList;

				contList = targetComp->get_owned_contexts();
				if(contList->length() > 0){
					RTC::LifeCycleState		stateResult;
cout << "count = " << contList->length() << endl;
					cout << "\n\n状態\t:";
					stateResult = contList[0]->get_component_state(targetComp._ptr());
//					stateResult = contList[0]->get_component_state(pTarget);
					switch(stateResult){
					case INACTIVE_STATE:
						cout << "INACTIVE" << endl;
						break;
					case ACTIVE_STATE:
						cout << "ACTIVE" << endl;
						break;
					case ERROR_STATE:
						cout << "ERROR" << endl;
						break;
					case CREATED_STATE:
						cout << "CREATED" << endl;
						break;
					default:
						cout << "UNKNOWN" << endl;
						break;
					}
				}

			}

			pTargetPorts = targetComp->get_ports();
			portMax = (int)(pTargetPorts->length());
			for(portCnt = 0; portCnt < portMax; portCnt++){
				portProfile = ((*pTargetPorts)[portCnt])->get_port_profile();

				// ポート名取得
				std::cout << "\tポート名\t: " << portProfile->name << std::endl;

				if(!(fullInfo)){
					propListNV = portProfile->properties;
					portProp = NVUtil::toProperties(propListNV);
					std::cout << "\tポート種別\t: " << portProp["port.port_type"] << std::endl;
					std::cout << "\tデータ型\t: " << portProp["dataport.data_type"] << std::endl;
					std::cout << "\t接続先ポート名\t: " << std::endl;
					connectList = portProfile->connector_profiles;
					for(unsigned int i = 0; i < connectList.length(); i++){
						for(unsigned int j = 0; j < connectList[i].ports.length(); j++){
							if(strcmp((char*)(portProfile->name), (char*)(connectList[i].ports[j]->get_port_profile()->name)) != 0){
								std::string		targetPath;

								targetPath = getRegisterNamingName(connectList[i].ports[j]->get_port_profile()->owner);
								std::cout << "\t\t\t" << targetPath << std::endl;

								// ポート名表示
								std::cout << "\t\t\t\t" << connectList[i].ports[j]->get_port_profile()->name << std::endl;
							}
						}
					}
				}
				else{
					std::cout << "\t\tプロパティ : " << std::endl;
					propListNV = portProfile->properties;
					portProp = NVUtil::toProperties(propListNV);
					for(unsigned int n = 0 ; n < propListNV.length(); n++){
						std::cout << "\t\t\t" << propListNV[n].name << " : " << portProp[(char *)(propListNV[n].name)] << std::endl;
					}

					std::cout << "\t\t接続情報 : " << std::endl;
					connectList = portProfile->connector_profiles;
					portProp = NVUtil::toProperties(propListNV);
					for(unsigned int i = 0; i < connectList.length(); i++){
						std::cout << "\t\t\t接続名 : " << connectList[i].name << std::endl;
						std::cout << "\t\t\t\t" << connectList[i].connector_id << std::endl;
						std::cout << "\t\t\tプロパティ : " << std::endl;
						propListNV = connectList[i].properties;
						for(unsigned int n = 0 ; n < propListNV.length(); n++){
							std::cout << "\t\t\t\t" << propListNV[n].name << " : " << portProp[(char*)(propListNV[n].name)] << std::endl;
						}

						std::cout << "\t\t\t接続先ポート名 : " << std::endl;
						for(unsigned int j = 0; j < connectList[i].ports.length(); j++){
							if(strcmp((char*)(portProfile->name), (char*)(connectList[i].ports[j]->get_port_profile()->name)) != 0){
								std::string		targetPath;

								targetPath = getRegisterNamingName(connectList[i].ports[j]->get_port_profile()->owner);
								std::cout << "\t\t\t\t" << targetPath << std::endl;

								// ポート名表示
								std::cout << "\t\t\t\t\t" << connectList[i].ports[j]->get_port_profile()->name << std::endl;
							}
						}
					}
				}
				std::cout << "\n\n" << std::endl;
			}
		}
	}

	return 1;
}

int referenceNamingServer(std::string command_str, std::vector<std::string> *cmnd_list)
{
	size_t serverCnt = 0;

	for(serverCnt = 1; serverCnt < (*cmnd_list).size(); serverCnt++){
		connectNamingServer((*cmnd_list)[serverCnt]);
	}

	return 1;
}

int quitApplication(std::string command_str, std::vector<std::string> *cmnd_list)
{
	// 起動したプロセスを終了させて永久ルールから抜ける
	exitComponent();
	return 0;
}

void listupComponentCallback(std::string name, std::string kind, int type, int layered, std::string path, CORBA::Object_ptr pComp)
{
	INSERT_NEST(layered);
	if(type & CALLBACK_TYPE_ZOMB)
		cout << name << "\t(" << kind << " zombie)" << endl;
	else if (type & CALLBACK_TYPE_SERV)
		cout << name << endl;
	else
		cout << name << "\t(" << kind << ")" << endl;
}
int listupComponents(std::string command_str, std::vector<std::string> *cmnd_list)
{
	// コンポーネント一覧表示
	std::cout << "\n" << std::endl;
	if(cmnd_list->size() == 2){
		if((*cmnd_list)[1] == CMD_PARAM_NAMING_LIST){
			listupNamingServers(command_str, cmnd_list);
		}
		else{
			enumNamingServers(listupComponentCallback, CALLBACK_TYPE_ALL, (*cmnd_list)[1]);
		}
	}
	else{
		enumNamingServers(listupComponentCallback, CALLBACK_TYPE_ALL);
	}
	std::cout << "\n" << std::endl;
	return 1;
}

void listupNamingServerCallback(std::string name, std::string kind, int type, int layered, std::string path, CORBA::Object_ptr pComp)
{
	cout << "\t" << name << endl;
}

int listupNamingServers(std::string command_str, std::vector<std::string> *cmnd_list)
{
	// ネーミングサーバ一覧表示
	//listNamingServer();
	cout << "\n" << endl;
	enumNamingServers(listupNamingServerCallback, CALLBACK_TYPE_SERV);
	cout << "\n" << endl;

	return 1;
}

int showCommandsList(std::string command_str, std::vector<std::string> *cmnd_list)
{
	command_table::iterator		it;

	std::cout << "\n" << endl;
	for(it = commandList.begin(); it != commandList.end(); it++){
		if(!(it->second.alias)){
			std::cout << "\t" << it->first << it->second.description << std::endl;
		}
	}
	std::cout << "\n" << std::endl;

	return 1;
}

#ifdef _DEBUG
void dumpIniFile()
{
	std::vector< std::string >::iterator it;

	cout << "********** Inifile Dump **********" << endl;
	for(it = defList.begin(); it != defList.end(); it++){
		cout << *it << endl;
	}
}

void dumpCompList()
{
	std::vector< std::string >::iterator it;

	cout << "********** CompList Dump **********" << endl;
	for(it = compList.begin(); it != compList.end(); it++){
		cout << *it << endl;
	}
}

void dumpMapInfo()
{
	def_file_table::iterator		it;

	cout << "********** InifileMap Dump **********" << endl;
	for(it = iniFileMap.begin(); it != iniFileMap.end(); it++){
		cout << it->first << endl;

		std::vector< std::string >::iterator it2;
		for(it2 = it->second.begin(); it2 != it->second.end(); it2++){
			cout << "\t" << *it2 << endl;
		}
	}
}

void dumpCmdLineInfo()
{
	command_line_table::iterator		it;

	cout << "********** CommandLineMap Dump **********" << endl;
	for(it = commandlineMap.begin(); it != commandlineMap.end(); it++){
		cout << it->first << endl;

		cout << "\t" << it->second << endl;
	}
}
#endif

/*
 * ちょっとした要望（メモ）
 *
 * ・コンポーネントリンクは、ネームサーバ指定された絶対パス指定時でも、ネームサーバを参照しないで接続出きるようにしたい。
 * 		コンポーネントがダブっている場合には、パス指定されているネームサーバのコンポーネントを使用することにし、通常はネームサーバを見ないようにする。
 * 		また、ネームサーバは参照コマンドを指定して、事前に参照対象に含めているが、絶対パス指定された場合には、自動で参照を追加するようにしたい。
 * ・
 */
