/*
 * RtcBatch.h
 *
 *  Created on: 2010/12/03
 *      Author: yotubata
 */

#ifndef RTCBATCH_H_
#define RTCBATCH_H_

#define DEF_BUFF_SIZE	512
#define LINK_SECTION		"LinkInfo"
#define START_COMP_SECTION	"StartComponent"
#define COMPONENT_LIST		"ComponentList"
#define RTC_CONF_FILENAME	"rtc.conf"
#define RTC_NAMINGSRV_KEY	"corba.nameservers:"

#define RTC_CONTEXT_NAME	"rtc"

#define CMD_PARAM_NAMING_LIST		"-naming"

#define ORIGINAL_CONF_KEY			"#!rtc_batch"
#define DEFAULT_CONFIG_FILE_NAME	"default.rbdf"

#define FULLAUTO_START_FLAG		"-fullauto"
#define PARTAUTO_START_FLAG		"-autostart"

#ifdef RTM_Ver4
#define CREATE_RTC_NAME(name)	name + "0.rtc"
#else
#define CREATE_RTC_NAME(name)	name + "0.rtc"
#endif

typedef int (*CommandProc)(std::string command_str, std::vector<std::string> *cmnd_list);
typedef std::string (*ShowCandidateProc)(std::string cmd);

struct CommandData{
	CommandData()	{alias = false; description = ""; content = ""; func = NULL; showCandidate = NULL;}
	bool				alias;
	std::string			description;
	std::string			content;
	CommandProc			func;
	ShowCandidateProc	showCandidate;
};

typedef std::map< std::string, std::vector<std::string> > def_file_table;
typedef def_file_table::value_type table_val;

typedef std::map< std::string, std::string > component_table;
typedef component_table::value_type comp_val;

typedef std::map< std::string, std::string > command_line_table;
typedef component_table::value_type command_line;

typedef std::map< std::string, CommandData > command_table;
typedef command_table::value_type command_val;

enum _auto_start_type{
	full_auto = 1,
	part_auto
};


#define INSERT_NEST(CNT)		for(int cnt = 0; cnt < CNT; cnt++)	std::cout << "\t"

/* 文字列操作関連 */
std::vector<std::string>	string_token(const std::string line, const char* tok);
std::vector<std::string>	commandLine_token(const std::string line);

//void listNamingServer();
//void componentList(RTC::CorbaNaming curNaming, CosNaming::NamingContext_ptr  curContext, int nest_cnt=0);

/* 起動情報ファイル関連 */
int readConnectInfoFile(const std::string fileName = DEFAULT_CONFIG_FILE_NAME);
int createIniFileTable();
int createComponentTable();
int createCommandLineTable();
int readRtcConf(const std::string rtc_conf = RTC_CONF_FILENAME);
int readConfigurationFile(int argc, char** argv);

/* コンポーネント関連 */
//void	listupComponent(const std::string serverName);
//int		startComponent();
//int		linkComponent();
int		exitComponent();
//int		deactivateComponent(std::string compName);
//int		activateComponent(std::string compName);


//int		componentControl(std::string compName, controlType contID, CORBA::Object_ptr pComponent = NULL);


#if 0
int		linkComponent(std::string srcComp, std::string destComp);
#ifdef RTM_Ver4
RTC::PortList_var	findComponentPort(std::string compPortName, int portType);
#else
RTC::PortService findComponentPort(std::string compPortName, int portType);
#endif
#endif

/* コマンド関連 */
void*	commandInput(void* param);
std::string					inputSupport(const char *buff);
std::vector<std::string>	candidateCommand(const std::string command = "");
void createCommandList();
int run_command(const std::string command);

// ネームサーバ追加
int referenceNamingServer(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
// ネームサーバ一覧表示
int listupNamingServers(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
// コンポーネント一覧表示
int listupComponents(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
// アプリケーション終了
int quitApplication(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
// ヘルプ表示
int showCommandsList(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);

int showComponentInfo(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int showComponentConfiguration(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int setComponentConfiguration(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int saveScriptFile(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int loadScriptFile(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int showScriptList(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int showHistoryList(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int runScript(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int deactivate_component(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int activate_component(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int exit_component(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);

int start_component(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int stop_component(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int reset_component(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);

int getCurrentLinkInfo(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int linkComponents(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);
int unlinkComponents(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);



int runComponent(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);


int test(std::string command_str = "", std::vector<std::string>  *cmnd_list = 0);


void listupNamingServerCallback(std::string name, std::string kind, int type, int layered = 0, std::string path = "", CORBA::Object_ptr pComp = NULL);
void listupComponentCallback(std::string name, std::string kind, int type, int layered = 0, std::string path = "", CORBA::Object_ptr pComp = NULL);
void listShowComponentCallback(std::string name, std::string kind, int type, int layered = 0, std::string path = "", CORBA::Object_ptr pComp = NULL);

void createLinkInfoCallback(std::string name, std::string kind, int type, int layered = 0, std::string path = "", CORBA::Object_ptr pComp = NULL);
void activateComponentCallback(std::string name, std::string kind, int type, int layered = 0, std::string path = "", CORBA::Object_ptr pComp = NULL);

std::string candidateComponent(const std::string command);
std::string candidateVoid(const std::string command);
std::string candidateNaming(const std::string command);
std::string candidateFile(const std::string command);
std::string candidateComponentAndServer(const std::string command, const int type);
std::string candidateComponentPort(const std::string command);
std::string candidateComponentConfig(const std::string command);



#endif /* RTCBATCH_H_ */
