/*
#include <sys/types.h>
#include <sys/wait.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <pthread.h>
#include <dirent.h>
#include <stdlib.h>
#include <fstream>

*/
#define _RTC_CONTROL_MAIN	1
#include "RtcControl.h"


#include <iostream>
#include <regex.h>

#include <rtm/CorbaNaming.h>
#include <rtm/CorbaConsumer.h>



#include "Tools.h"




/**
 * @brief 参照するネーミングサーバを追加する
 * @return 0 : 追加正常
 */
int connectNamingServer(const std::string serverName)
{
	std::string					naming_server;
	std::vector<std::string>	paramList;

	paramList = string_token(serverName, " :");
	if(paramList.size() == 2)
		naming_server = paramList[0] + ":" + paramList[1];
	else
		naming_server = paramList[0];
	std::cout << "ORB_NAMESERVER = " << naming_server << std::endl;

	try{
		// ORBネーミングサーバオブジェクト取得
		RTC::CorbaNaming name_s(corbaORB, naming_server.c_str());
		namingServerList.insert(corba_naming_list(naming_server, name_s));
	}
	catch(...){
		std::cout << "Name server not found!" << std::endl;
	}

	return 0;
}

std::string	getNamingServerName(std::string namingServer)
{
	std::string	result;
	regex_t		reg;
	regmatch_t	match;

	// 正規表現を用いて、変な書式のネーミングサーバ名を本来の状態に変更する
	/* 変な例
	 * corbaloc::localhost/NameService
	 * corbaloc::localhost:2809/NameService
	 */
	// 正規表現の種(パターン)を準備する
	if(regcomp(&reg, "\\w::.+/NameService", REG_ICASE | REG_EXTENDED | REG_NEWLINE) != 0) {
		std::cout << "パターン構築失敗" << std::endl;
	}
	// パターンマッチングを行う
	if(regexec(&reg, namingServer.c_str(), 1, &match, 0) == 0) {
		// 必要な文字列を取り出す
		std::cout << "パターン一致" << std::endl;
	}
	else{
		result = namingServer;
	}
	// 種を開放する
	regfree(&reg);
	return result;
}

/**
 * @brief コンポーネントを検索する
 * @param compName		コンポーネント名
 * @param namingServer	ネーミングサーバー名
 * @param port			ネーミングサーバの使用ポート
 * @return コンポーネントオブジェクトのポインタ
 */
CORBA::Object_ptr findComponent(const std::string compInstName, const CORBA::Object_ptr obj, const bool showCandidateComp)
{
	std::string					tempCompPath = "";
	std::vector< std::string >	compPath;
	CORBA::Object_ptr			pResult = 0;

	// ネームサーバ付きで指定されている場合もあるので、分解する
	compPath = string_token(compInstName, "/");

//	findCompName = compInstName;
	findCompName = compPath[compPath.size() -1];
	findCompPath = "";
	for(std::vector< std::string >::iterator it = compPath.begin(); it != compPath.end(); it++){
		findCompPath = tempCompPath;
		if(tempCompPath.length() > 0)	tempCompPath = tempCompPath + "/";
		tempCompPath = tempCompPath + *it;
	}

	component_list.clear();
	enumNamingServers(findComponentCallback, CALLBACK_TYPE_COMP);
	if(component_list.size() == 1){
		pResult = component_list[0].pPrt;
	}
	else{
		// 同一コンポーネント名が複数あるので、ネームサーバ付きの名前を表示する。
		// objが指定されている場合はobjを探す
		for(std::vector< ComponentData >::iterator it = component_list.begin(); it != component_list.end(); it++){
			if(showCandidateComp)
				std::cout << "\t" << it->path << "/" << it->name << std::endl;
			if(obj != NULL){
				if(obj->_is_equivalent(it->pPrt)){
					pResult = it->pPrt;
					break;
				}
			}
		}
	}

	return pResult;
}

void enumNamingServers(const EnumCallbackProc func, const short callbackType, const std::string serverName)
{
	// ネーミングサーバ分回る
	for(corbaNamingList::iterator itNamingServer = namingServerList.begin(); itNamingServer != namingServerList.end(); itNamingServer++){
		if((serverName.length() == 0) || (itNamingServer->first == serverName)){
			// ネーミングサーバー名通知？
			if(callbackType & CALLBACK_TYPE_SERV){
				func(itNamingServer->first, "", CALLBACK_TYPE_SERV, 0, "", NULL);
				//std::cout << itNamingServer->first << std::endl;
			}
			if(callbackType & 0x00000007){
				enumCosNaming(func, callbackType, itNamingServer->first, itNamingServer->second, itNamingServer->second.getRootContext());
			}
		}
	}
}

void enumCosNaming(const EnumCallbackProc func, const short callbackType, const std::string namingName, RTC::CorbaNaming curNaming, CosNaming::NamingContext_ptr curContext, int layered)
{
	CosNaming::NamingContext_ptr	nextContext;
	CosNaming::BindingList_var		bindList;
	CosNaming::BindingIterator_var	bindListIt;
	CORBA::Object_ptr				pComp = NULL;
	unsigned long					contMax = 0;
	std::string						nextNamingName;

	// コンテキストが無い場合は処理しない
//	std::cout << "enumCosNaming called!" << std::endl;
	if(curContext == 0)		return;

	layered++;
	// コンテキストに格納されている要素を取得する
	// ほんとはbindListItを利用したいのだが、end()の位置を特定できそうにないのでちょっと強引な方法を使う事にする。
	do{
		contMax = contMax + 25;
		try{
			curNaming.list(curContext, contMax, bindList, bindListIt);
		}catch(...){
			std::cout << "エラーしちゃった" << std::endl;
			return;
		}
//		std::cout << "list size = " << bindList->length() << std::endl;
	}while(contMax == bindList->length());

	// 取得した要素の検証を行う
	for(size_t bindCnt = 0; bindCnt < bindList->length(); bindCnt++){
		for(size_t nameCnt = 0; nameCnt < bindList[bindCnt].binding_name.length(); nameCnt++){
			// 要素種別検証
			if(bindList[bindCnt].binding_type == CosNaming::nobject){
					// コンテキストではない
					// コンテキストと名称(id+kind)でコンポーネントのポインタを取得する
					try{
//						pComp = findComponent((char *)(bindList[bindCnt].binding_name[nameCnt].id), (char *)(bindList[bindCnt].binding_name[nameCnt].kind), curNaming.getNameServer());
						pComp = curContext->resolve(bindList[bindCnt].binding_name);
						// コンポーネントが見つかった
						if(pComp != NULL){
							if(pComp->_non_existent()){
								if(callbackType & CALLBACK_TYPE_ZOMB){
									func((char*)(bindList[bindCnt].binding_name[nameCnt].id), (char*)(bindList[bindCnt].binding_name[nameCnt].kind), CALLBACK_TYPE_ZOMB, layered, namingName, pComp);
								}
								//std::cout << bindList[bindCnt].binding_name[nameCnt].id << "\t(!! zombie !!)" << std::endl;
							}
							else{
								if(callbackType & CALLBACK_TYPE_COMP){
									func((char*)(bindList[bindCnt].binding_name[nameCnt].id), (char*)(bindList[bindCnt].binding_name[nameCnt].kind), CALLBACK_TYPE_COMP, layered, namingName, pComp);
								}
								//std::cout << bindList[bindCnt].binding_name[nameCnt].id << "\t(" << bindList[bindCnt].binding_name[nameCnt].kind << ")" << std::endl;
							}
						}
					}
					catch(...){
						/* ゴーストコンポーネントは表示しないことにする 2010/11/26
						std::cout << bindList[bindCnt].binding_name[nameCnt].id << "\t( unknown )" << std::endl;
						*/
					}
			}
			else if(bindList[bindCnt].binding_type == CosNaming::ncontext){
				// ホストコンテキスト
				if(callbackType & CALLBACK_TYPE_CNTX){
					func((char*)(bindList[bindCnt].binding_name[nameCnt].id), (char*)(bindList[bindCnt].binding_name[nameCnt].kind), CALLBACK_TYPE_CNTX, layered, namingName, NULL);
				}
				nextNamingName = namingName + "/" + (char*)(bindList[bindCnt].binding_name[nameCnt].id);
				nextContext = curNaming.bindOrResolveContext(curContext, bindList[bindCnt].binding_name, curNaming.newContext());
				if(nextContext != NULL){
					//std::cout << bindList[bindCnt].binding_name[nameCnt].id << "\t(" << bindList[bindCnt].binding_name[nameCnt].kind << ")" << std::endl;
					enumCosNaming( func, callbackType, nextNamingName, curNaming, nextContext, layered);
				}
			}
			else{
				// その他のよく分からないもの
				//func(bindList[bindCnt].binding_name[nameCnt].id, bindList[bindCnt].binding_name[nameCnt].kind, 0, layered, namingName);
				std::cout << bindList[bindCnt].binding_name[nameCnt].id << "\t(" << bindList[bindCnt].binding_name[nameCnt].kind << ")" << std::endl;
			}
		}
	}
}

int componentControl(std::string compName, controlType contID, CORBA::Object_ptr pComponent)
{
	int	result = 0;

	// CORBA オブジェクトの取得
	CORBA::Object_ptr pComp;
	try{
		if(pComponent == NULL){
			pComp = findComponent(compName.c_str());
		}
		else{
			pComp = pComponent;
		}
		if(pComp != NULL){
			RTC::ReturnCode_t ret;

			// コンポーネントが見つかった
			RTC::CorbaConsumer<RTC::RTObject>		compObj;
			RTC::ExecutionContextList_var			contList;

			compObj.setObject(pComp);

			// RTCコンテキストの取得
#ifdef RTM_Ver4
			contList = compObj->get_contexts();
#else
			contList = compObj->get_owned_contexts();
#endif
			// コンテキスト数の確認 (正常だと１になる)
			if(contList->length() > 0){
				std::string	compRealName;
				compRealName = compObj->get_component_profile()->instance_name;
				switch(contID){
					case cont_type_activate_comp:
						ret = contList[0]->activate_component(compObj._ptr());
						std::cout << compRealName << " をアクティベートしました。" << std::endl;
						break;
					case cont_type_deactivate_comp:
						ret = contList[0]->deactivate_component(compObj._ptr());
						std::cout << compRealName << " をデアクティベートしました。" << std::endl;
						break;
					case cont_type_start_comp:
						ret = contList[0]->start();
						break;
					case cont_type_stop_comp:
						ret = contList[0]->stop();
						break;
					case cont_type_reset_comp:
						ret = contList[0]->reset_component(compObj._ptr());
						break;
					case cont_type_exit_comp:
						compObj->exit();
						break;
					default:
						break;
				}
			}
		}
	}
	catch(...){
		/*
		 * コンポーネントが登録されていない・・・
		 * 起動に失敗したらしい
		 */
	}

	return result;
}


RTC::PortService_ptr getPortInfo(std::string compName, std::string portName)
{
	RTC::PortService_ptr	result = NULL;
	CORBA::Object_ptr	pTarget;

	pTarget = findComponent(compName);
	if(pTarget != NULL){
		RTC::CorbaConsumer<RTC::RTObject>		targetComp;
		RTC::PortServiceList					*pTargetPorts;
		RTC::PortProfile						*portProfile;
		int										portCnt, portMax;

		SDOPackage::NVList	propListNV;
		RTC::Properties		portProp;
		std::string			portType;

		RTC::ConnectorProfileList	connectList;
		RTC::ComponentProfile		*compProfile;

		// RTCオブジェクトにする
		targetComp.setObject(pTarget);

		// RTCプロファイル情報取得
		compProfile = targetComp->get_component_profile();
		// ポート情報取得
		pTargetPorts = targetComp->get_ports();
		// ポート数取得
		portMax = (int)(pTargetPorts->length());
		// ポート数分ループ
		for(portCnt = 0; portCnt < portMax; portCnt++){
			result = ((*pTargetPorts)[portCnt]);
			// ポートプロファイル取得
			portProfile = ((*pTargetPorts)[portCnt])->get_port_profile();
			// ポート名確認
			if(strcmp((char*)(portProfile->name), portName.c_str()) == 0)	break;
		}
	}
	return result;
}


bool isConnected(RTC::PortService_ptr src, RTC::PortService_ptr dest)
{
	bool					result = false;
	RTC::PortProfile				*portProfile;
	RTC::ConnectorProfileList	connectList;

	// ポートプロファイル取得
	portProfile = src->get_port_profile();
	// 既接続情報取得
	connectList = portProfile->connector_profiles;
	for(unsigned int i = 0; i < connectList.length(); i++){
		for(unsigned int j = 0; j < connectList[i].ports.length(); j++){
			if(dest->_is_equivalent(connectList[i].ports[j])){
				result = true;
				/*
				std::string		targetPath;

				targetPath = getRegisterNamingName(connectList[i].ports[j]->get_port_profile()->owner);
				std::cout << "\t\t\t" << targetPath << std::endl;

				// ポート名表示
				std::cout << "\t\t\t\t" << connectList[i].ports[j]->get_port_profile()->name << std::endl;
				*/
			}
		}
	}

	return result;
}

std::string getPortProperty(RTC::PortService_ptr pPort, std::string propName)
{
	std::string			result = "";
	RTC::PortProfile			*portProfile;
	SDOPackage::NVList	propListNV;
	RTC::Properties		portProp;

	// ポートプロファイル取得
	portProfile = pPort->get_port_profile();

	// ポートプロパティ取得
	propListNV = portProfile->properties;
	portProp = NVUtil::toProperties(propListNV);

	result = portProp[propName];

	return result;
}

std::string getPortDataType(RTC::PortService_ptr pPort)
{
	return getPortProperty(pPort, "dataport.data_type");
}

std::string getPortType(RTC::PortService_ptr pPort)
{
	return getPortProperty(pPort, "port.port_type");
}
std::string getRegisterNamingName(CORBA::Object_ptr pComp)
{
	RTC::CorbaConsumer<RTC::RTObject>	targetComp;
	RTC::ComponentProfile				*compProfile;
	std::string						result = "";

	targetComp.setObject(pComp);
	compProfile = targetComp->get_component_profile();
	findCompName = (char*)(compProfile->instance_name);

	component_list.clear();
	enumNamingServers(findComponentCallback, CALLBACK_TYPE_COMP);

	for(std::vector< ComponentData >::iterator it = component_list.begin(); it != component_list.end(); it++){
		if(pComp->_is_equivalent(it->pPrt)){
			result = it->path +  "/"  + it->name;
			break;
		}
	}

	return result;
}
void findComponentCallback(std::string name, std::string kind, int type, int layered, std::string path, CORBA::Object_ptr pComp)
{
	ComponentData	data;
//cout << findCompName << " / " << name << endl;
	if(findCompName.compare(name) == 0){
		if((findCompPath.length() == 0) || (findCompPath.compare(path) == 0)){
			data.name = name;
			data.path = path;
			data.pPrt = pComp;
			component_list.push_back(data);
		}
	}
}

int linkComponent(std::string srcComp, std::string srcPort, std::string destComp, std::string destPort,
						std::string name, std::string if_type, std::string df_type, std::string ss_type)
{
	RTC::PortService_ptr	srcInfo, destInfo;

	// 接続元ポート情報取得
/*
	std::cout << "\t[" << srcComp << "]の<" << srcPort << ">src" << std::endl;
	std::cout << "\t[" << destComp << "]の<" << destPort << ">dest" << std::endl;
*/
	srcInfo = getPortInfo(srcComp, srcPort);
	if(srcInfo == 0){
		std::cout << "\t[" << srcComp << "]の<" << srcPort << ">ポートは存在しません。" << std::endl;
	}
	// 接続先ポート情報取得
	destInfo = getPortInfo(destComp, destPort);
	if(destInfo == 0){
		std::cout << "\t[" << destComp << "]の<" << destPort << ">ポートは存在しません。" << std::endl;
	}

	// 接続済み確認
	if(isConnected(srcInfo, destInfo)){
		std::cout << "接続済み" << std::endl;
		return 1;
	}

	// ポートデータ型確認
	if(getPortDataType(srcInfo).compare(getPortDataType(destInfo)) != 0){
		std::cout << "データ型が一致していません" << std::endl;
		return 1;
	}

	// InPort / OutPort or Corba確認
	if(getPortType(srcInfo).compare("DataInPort") == 0){
		// 相手はOutPort
		if(getPortType(destInfo).compare("DataOutPort") != 0){
			std::cout << "ポートタイプが一致しません。 InPortにOutPort以外を接続しようとしました。\n\t[" << getPortType(destInfo) << "]" << std::endl;
			return 1;
		}
	}else if(getPortType(srcInfo).compare("DataOutPort") == 0){
		// 相手はInPort
		if(getPortType(destInfo).compare("DataInPort") != 0){
			std::cout << "ポートタイプが一致しません。 OutPortにInPort以外を接続しようとしました。" << std::endl;
			return 1;
		}
	}else{
		// 相手はCorba
		if(getPortType(destInfo).compare("CorbaPort") != 0){
			std::cout << "ポートタイプが一致しません。 CorbaPortにCorbaPort以外を接続しようとしました。" << std::endl;
			return 1;
		}
	}

	// ポートを接続する
	{
		RTC::ReturnCode_t		ret;
		RTC::ConnectorProfile	prof;

		prof.connector_id = "";
		if(name.length() == 0){
			name = srcPort + "_" + destPort;
		}
		prof.name = CORBA::string_dup(name.c_str());
		prof.ports.length(2);

		prof.ports[0] = srcInfo;
		prof.ports[1] = destInfo;

		if(getPortType(srcInfo) != "CorbaPort"){
			/*
			 * Ver1.0 からは
			 * InterfaceType	= corba_cdr
			 * DatafllowType	= push	/ pull
			 * SubscriptionType	= flush	/ new	/ periodic
			 *
			 */
			CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.interface_type", if_type.c_str()));
			CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.dataflow_type", df_type.c_str()));
			CORBA_SeqUtil::push_back(prof.properties, NVUtil::newNV("dataport.subscription_type", ss_type.c_str()));
		}
		ret = srcInfo->connect(prof);
		// おまじない！
		sleep(2);
		if(ret == RTC::RTC_OK)
			std::cout << "接続できたと思います。" << std::endl;
		else
			std::cout << "失敗したと思います。" << std::endl;
	}

	return 1;
}
