#include <stdio.h>
#include <stdlib.h>
#include <map>
#include <iostream>
#include <fstream>
#include <string.h>

#include "QueryInfo.h"

#ifdef _DEBUG
FILE		*pDbgLog;
#endif

#ifdef _DEBUG
void dump_post_data(unsigned char *buf,int len){
    unsigned char *p,*q;
    unsigned char *end   = &buf[len-1];
    int i = 0;
    unsigned char str[33];

    fprintf(pDbgLog, "<h2>Method POST multipart/form-data DUMP</h2>\n");
    for (p = buf; p <= end; p++){
        if (i < 32){
            if ( *p < 0x10)
                fprintf(pDbgLog, "0%x ",*p);
            else
                fprintf(pDbgLog, "%x ",*p);
            if ( *p < ' ' || 0x7E < *p)
                str[i] = '.';
            else
                str[i] = *p;
            i++;
            }
        else{
            str[i] = '\0';
            fprintf(pDbgLog, "| ");
            for (q = str; *q; q++){
                if ( *q == '<')
                    fprintf(pDbgLog, "&lt;");
                else if ( *q == '>')
                    fprintf(pDbgLog, "&gt;");
                else if ( *q == '&')
                    fprintf(pDbgLog, "&amp;");
                else if ( *q == '\\')
                    fprintf(pDbgLog, "&quot;");
                else
                    fprintf(pDbgLog, "%c", *q);
                }
            fprintf(pDbgLog, "<br />\n");
            i = 0;
            p--;
            }
        }
    str[i] = '\0';
    for ( ; i < 32; i++){
        fprintf(pDbgLog, "...");
        }
    fprintf(pDbgLog, "| %s<br />\n",str);
    }
#endif

QueryInfo::QueryInfo()
{
	trans_data.fileData				= 0;
	trans_data.fileSize				= 0;

#ifdef _DEBUG
	if(pDbgLog == NULL){
		pDbgLog = fopen("debug2.log", "a");
	}
#endif
}

QueryInfo::~QueryInfo()
{
	if(trans_data.fileData != 0){
		free(trans_data.fileData);
	}
}

void QueryInfo::ReadQueryData()
{
	std::string	query_str	= "";
	/*
	 * "CONTENT_LENGTH"を取り出す
	 * 環境変数にあるGET情報("QUERY_STRING")を読み込む
	 * GET情報が無ければ、POST情報を取得する
	 * 何もないこともあるかもね
	 */

#ifdef _DEBUG
		fprintf(pDbgLog, "リクエストデータ解析 (%s)\n", getenv("REMOTE_ADDR"));
#endif

	if(getenv("REMOTE_ADDR") != NULL){
		remote_ip = getenv("REMOTE_ADDR");
	}
	else{
		remote_ip = "unknown";
	}

	if(getenv("REQUEST_METHOD") == NULL){
		return;
	}

	if(strcmp(getenv("REQUEST_METHOD"), "GET") == 0){
		char		*pGetStr = 0;

		// GET or POST情報ありなので情報取得を行う
		pGetStr = getenv("QUERY_STRING");
		if(pGetStr != 0){
			query_str = pGetStr;
#ifdef _DEBUG
		fprintf(pDbgLog, "GET = %s\n", pGetStr);
		fflush(pDbgLog);
#endif
		}
	}
	else{
			// POST情報を見る
#ifdef _DEBUG
		fprintf(pDbgLog, "POST取得\n");
		fflush(pDbgLog);
#endif
		query_str = GetPostData();
	}

	if(query_str.length() > 0){
		// CGIリクエストが含まれている
		ParseQuery(query_str);
	}
#ifdef _DEBUG
	fprintf(pDbgLog, "判定終了\n");
	fflush(pDbgLog);
#endif
}

void QueryInfo::ParseQuery(std::string query_value)
{
	// Key名・値のペアでデータを格納する
	size_t	div_pos, query_pos, cur_index = 0;
	std::string	key_name, key_value;
	std::string query_str;

//	query_value = DecordQuery(query_value);
#ifdef _DEBUG
	fprintf(pDbgLog, "%s\n", query_value.c_str());
	fflush(pDbgLog);
#endif
	query_data.clear();

	for(query_pos = query_value.find_first_of('&', cur_index);
			cur_index != std::string::npos;
			query_pos = query_value.find_first_of('&', cur_index)){

		if(query_pos != std::string::npos){
			query_str = query_value.substr(cur_index, query_pos - cur_index);
			cur_index = query_pos + 1;
		}
		else{
			query_str = query_value.substr(cur_index, query_pos);
			cur_index = query_pos;
		}
		div_pos = query_str.find_first_of('=');
		if(div_pos != std::string::npos){
			// 分割点が見つかった
			key_name = DecordQuery(query_str.substr(0, div_pos));
#ifdef _DEBUG
			fprintf(pDbgLog, "key = %s\t", key_name.c_str());
#endif
			key_value = DecordQuery(query_str.substr(div_pos + 1));
#ifdef _DEBUG
			fprintf(pDbgLog, "value = %s\n", key_value.c_str());
#endif
		 	query_data.insert(query_pair(key_name, key_value));
		}
	}
}

std::string QueryInfo::DecordQuery(std::string query_str)
{
	int			max_loop;
	char		*work = 0, *end_pos;
	char		dec_char[4];
	std::string	result;

	/*
	 * '+'(0x2b)はスペース(0x20)に
	 * %エスケープは後続文字を16進扱いに
	 * 0x0D 0x0A は 0x0Aに統一
	 */
	max_loop = query_str.length();
	work = (char*)malloc(max_loop * 3);
	if(work != 0){
		int		cur_index = 0, dec_index = 0;
		char	cur_char;

		memset(work, 0, max_loop * 3);
		while(cur_index < max_loop){
			cur_char = query_str.at(cur_index);
			if(cur_char == '+'){
				work[dec_index] = ' ';
				dec_index++;
				cur_index++;
				continue;
			}
			if(cur_char == '%'){
				// 後続文字を16進化する
				memset(dec_char, 0, sizeof(dec_char));
				dec_char[0] = query_str.at(cur_index + 1);
				dec_char[1] = query_str.at(cur_index + 2);
				work[dec_index] = (char)(strtol(dec_char, &end_pos, 16));
				dec_index++;
				cur_index = cur_index + 3;
				continue;
			}
			work[dec_index] = cur_char;
			dec_index++;
			cur_index++;
		}
		result = work;
		free(work);
	}
	return result;
}

std::string QueryInfo::GetFunction(std::string key)
{
	std::string	result;

	result = query_data[key];
	if(result.length() == 0){
		result = "default";
	}
	return result;
}

std::string QueryInfo::GetCommandMode(std::string key)
{
	std::string	result;

	result = query_data[key];
	if(result.length() == 0){
		result = "";
	}
	return result;
}
/*
std::string QueryInfo::GetFunctionName()
{
	std::string	result;

	result = query_data["Function"];
	if(result.length() == 0){
		result = "default";
	}
	return result;
}

std::string QueryInfo::GetSubFunctionName()
{
	std::string	result;

	result = query_data["SubFunction"];
	if(result.length() == 0){
		result = "default";
	}
	return result;
}

std::string QueryInfo::GetMode()
{
	std::string	result;

	result = query_data["Mode"];
	if(result.length() == 0){
		result = "";
	}
	return result;
}

std::string QueryInfo::GetCommand()
{
	std::string	result;

	result = query_data["Command"];
	if(result.length() == 0){
		result = "";
	}
	return result;
}
*/

std::string QueryInfo::GetQueryPair(std::string key)
{
	std::string	result;

	result = query_data[key];
	if(result.length() == 0){
		result = "";
	}
	return result;
}

std::string QueryInfo::GetPostData()
{
	char		*buff;
	int			content_len = 0, read_size;
	std::string	result = "";

	if(getenv("CONTENT_LENGTH") != 0){
		content_len = atoi(getenv("CONTENT_LENGTH"));
	}
/*
	// 標準入力からデータを取り出す
	cin >> result;
*/

#ifdef _DEBUG
	fprintf(pDbgLog, "コンテント長 = %d\n", content_len);
#endif

	if(content_len > 0){
		buff = (char*)malloc(content_len);
		read_size = fread(buff, 1, content_len, stdin);

#if _DEBUG
		{
			FILE	*testData;

			testData = fopen("test.dat", "ab");
			fwrite(buff, 1, read_size, testData);
			fclose(testData);
		}
#endif
/*
		buff[read_size] = 0;
		buff[read_size + 1] = 0;
		buff[read_size + 2] = 0;
		buff[read_size + 3] = 0;
*/
		if(strcasestr(getenv("CONTENT_TYPE"), "multipart")){
			// "multipart"の場合はファイル転送が行われている

#ifdef _DEBUG
			fprintf(pDbgLog, "maltipart(%s, %s)\n", getenv("CONTENT_TYPE"), getenv("HTTP_CONTENT_TYPE"));
			fflush(pDbgLog);
#endif

			GetMultipartData((unsigned char *)buff, content_len);
//			dump_post_data((unsigned char *)buff,content_len);
		}
		else{
			result = buff;
		}
		free(buff);
	}
	return result;
}

void QueryInfo::GetMultipartData(unsigned char *buff, int len)
{
	std::string	key_name, key_value;
	std::string	env_data = getenv("CONTENT_TYPE");
	std::string boundary_str;
	size_t		boundary_pos;
	char		*boundary_data = 0;
	char		*post_value;
	char		*key_pos = 0, *key_end = 0, *file_trans = 0;

	int			i;
	range_st	range;
	size_t		start, max_len = len;

	/*
	 * パート分解識別子を取り出す
	 * "Content-Disposition: form-data; "を探す
	 * 後続の name=を探す、さらに後続の"??????"を取り出す
	 * さらに、"Content-Type: を探す
	 */

#ifdef _DEBUG
fprintf(pDbgLog, "QueryInfo::GetMultipartData\n");
fflush(pDbgLog);
#endif

	boundary_pos = env_data.find(_BOUNDARY_STR);
	if(boundary_pos != std::string::npos){
		boundary_str = "--" + env_data.substr(boundary_pos + strlen(_BOUNDARY_STR));
	}
	else{
		// multipartじゃなかった
#ifdef _DEBUG
fprintf(pDbgLog, "multipartじゃなかった\n");
fflush(pDbgLog);
#endif
		return;
	}
	post_value = (char*)malloc(len);
	memset(post_value, 0, len);
	boundary_data = (char*)malloc(boundary_str.length() + 1);
	memset(boundary_data, 0, boundary_str.length() + 1);
	memcpy(boundary_data, boundary_str.c_str(), boundary_str.length());

	i = 0;
	start = 0;
	while(1){
		file_trans = 0;
		range = GetPartRange(start, boundary_data, buff, max_len);
		if( (range.start == 0) && (range.end == 0)){
#ifdef _DEBUG
fprintf(pDbgLog, "Multipartデータなし\n");
fflush(pDbgLog);
#endif
			break;	// 開始終了が０ならデータなし
		}

		// データブロック抜きだし
		memset(post_value, 0, len);
		memcpy(post_value, &(buff[range.start]), range.end - range.start);

		if(memcmp("Content-Disposition: form-data; ", post_value, strlen("Content-Disposition: form-data; ")) == 0){
			// データフィールド検出
			//
			i = strlen("Content-Disposition: form-data; ");
			/*
			 * name=
			 * filename=
			 *
			 * Content-Type: ?????/?????
			 *
			 * 改行コードが2連続発生後実際のデータ
			 */
			// name= を探す
			key_pos = strstr(&(post_value[i]), "name=");
			if(key_pos != 0){
				// name=が見つかったので、取り出す
				if(key_pos[strlen("name=")] == '\"'){
					key_end = strchr(&(key_pos[strlen("name=") + 1]), '\"');
					*key_end = 0;
					key_name = &(key_pos[strlen("name=") + 1]);
					key_end++;
					file_trans = strstr(key_end, "filename=");
				}
				else{
					// "がない場合は・・・ないかな？
				}

				// 付属しているはずのバイナリを探す
				key_pos = strstr(key_end, "\r\n\r\n");
				if(key_pos == 0){
					key_pos = strstr(key_end, "\n\n");
				}
				if(key_pos != 0){
					// バイナリ抜きだし
					if(*key_pos == '\r'){
						// 4バイト消費
						key_pos = key_pos + 4;
					}
					else{
						// 2バイト消費
						key_pos = key_pos + 2;
					}
					if(file_trans == 0){
						key_value = key_pos;
					}
					else{
						// ファイルの場合
						trans_data.fileSize = (&(post_value[range.end - range.start])) - (key_pos);
						trans_data.fileData = (unsigned char*)malloc(trans_data.fileSize);
						memcpy(trans_data.fileData, key_pos, trans_data.fileSize);
						key_value = "TransFileData";
#ifdef _DEBUG
					fprintf(pDbgLog, "TransFileSize = %d\n", trans_data.fileSize);
fflush(pDbgLog);
#endif
					}
#ifdef _DEBUG
					fprintf(pDbgLog, "key = %s\t", key_name.c_str());
					fprintf(pDbgLog, "value = %s\n", key_value.c_str());
fflush(pDbgLog);
#endif
				 	query_data.insert(query_pair(key_name, key_value));
				}
			}
		}
		start = range.end;
	}

	free(post_value);
	free(boundary_data);
}

range_st QueryInfo::GetPartRange(size_t start, char* boundary_data, unsigned char* buff, int max_len)
{
	int			i;
	range_st	result = {0,0};
	size_t		*pos;

	i = start;

	if((int)(i + strlen(boundary_data) * 2) > max_len){
		return result;
	}

	pos = &(result.start);
	while((int)i < max_len){
		if(memcmp(boundary_data, &(buff[i]), strlen(boundary_data)) == 0){
			if(pos == &(result.end)){
				// 終端検索状態
				// 終端には0d0aが入っているので除去しないと・・・
				if(buff[i-1] == 0x0a)	i--;
				if(buff[i-2] == 0x0d)	i--;
				i--;
				*pos = i;
				break;
			}

			i = i + strlen(boundary_data);
			/*
			 * きっと0x0d or 0x0a が続いてる・・・
			 */

			for( ; i < max_len ; i++){
				if((buff[i] != 0x0d) && (buff[i] != 0x0a)){
					*pos = i;
					pos = &(result.end);
					break;
				}
			}
		}
		i++;
	}

	return result;
}
