/*
 * SystemStatus.h
 *
 *  Created on: 2011/01/13
 *      Author: yotubata
 */

#ifndef SYSTEMSTATUS_H_
#define SYSTEMSTATUS_H_

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <string>
#include <vector>
#include <map>

#define DATA_CONT_ID	0x4c564443

typedef std::map< long, std::string >	status_str;
typedef status_str::value_type			status_pair;
typedef status_str::iterator			status_it;

typedef std::map< long, long >		status_value;
typedef status_value::value_type		value_pair;
typedef status_value::iterator			value_it;

struct ErrorInfo
{
	long	errStrID;
	long	notifyInfo;
	std::vector< std::string >		errRecoveryURL;
};

typedef std::map< long, ErrorInfo >	error_value;
typedef error_value::value_type		error_pair;
typedef error_value::iterator			error_it;

enum TerminalType{
	TType_Control		= 0x01000000,	// 	制御端末
	TType_Frontdesk		= 0x02000000,	// 	受付端末
	TType_Meeting		= 0x03000000,	// 	会議室端末
	TType_PA10			= 0x04000000,	// 	PA10
	TType_RH			= 0x05000000	// 	RH
};

enum StateType{
	SType_Normal		= 0x00000000,	//	正常
	SType_Wait			= 0x00000001,	//	待機
	SType_Warning		= 0x00007000,	//	警告
	SType_Instruction	= 0x00008000,	//	指示待ち
	SType_Error			= 0x0000F000	//	エラー
};

enum SystemStatus{
	SSTATE_NORMAL	= 0,
	SSTATE_WAIT		= 0,
	SSTATE_BOOT,
	SSTATE_RECEIPTING,
	SSTATE_CONTACTING,
	SSTATE_SERV_OPERATE_START,
	SSTATE_SERVING,
	SSTATE_DISPOSE_OPERATE_START,
	SSTATE_DISPOSING,
	SSTATE_MOVE_TO_MET,
	SSTATE_MOVE_TO_PA10,
	SSTATE_MOVE_TO_TRASH,
	SSTATE_MOVE_TO_PARK,
	SSTATE_DRINK_SET_STANDBY
};


struct CurrentStatusInfo
{
	char			scenarioID[16];			// 現在有効なシナリオ
	char			scenarioFileName[256];	// 現在実行中のシナリオファイル名
	unsigned long	runPosition;			// 実行中シナリオ位置
//	SystemStatus	systemStatus;			// 現在のシステム状態
	unsigned long	systemStatus;			// 現在のシステム状態
	unsigned long	terminalStatus[10];		// 各端末・ロボット(受付・会議室・PA10・RH)の状態
};

class SystemDataMgr
{
public:
	SystemDataMgr();	// 共有メモリ作成・アタッチ
	~SystemDataMgr(); // 共有メモリデタッチ・破棄

public:
	CurrentStatusInfo*	createData();
	bool				clearData();
	bool				destroyData();
	bool				isNewScenario();

	bool	getStatus(CurrentStatusInfo *info);
	bool	setStatus(CurrentStatusInfo state);
	bool	setScenarioID(std::string scenarioID);
	bool	setScenarioFileName(std::string scenarioFileName);
	bool	setScenarioPosition(unsigned long index);
	bool	setSystemStatus(SystemStatus status);
//	bool	setTerminalType(TerminalType type);

	std::string getErrorString(long id);
	std::string	getStatusString(long id);
	std::string	getRecoveryString()			{ return recoveryURL; }
	bool		writeSystemStateLog(std::vector< long > stateData)	{ return WriteLogData(stateData); }

	bool WriteLogData(std::vector< long > stateData);

private:
	void readStrList(const char** list, status_str* map_buff);
	void readStatusList(const char** list);
	void readErrorList(const char** list);

private:
	CurrentStatusInfo	*shmpt;
	status_str			statusStrList;
	status_str			errorStrList;
	status_value		statusList;
	error_value			errorList;

	long				systemStatus;
	long				errorTerminal;
	std::string			recoveryURL;
};

#endif /* SYSTEMSTATUS_H_ */
