// -*-C++-*-
/*!
 * @file  ControlTerminalSVC_impl.h
 * @brief Service implementation header of ControlTerminal.idl
 *
 */

#include "ControlTerminalSkel.h"
#include "TerminalServer.h"

#ifndef CONTROLTERMINALSVC_IMPL_H
#define CONTROLTERMINALSVC_IMPL_H

class TerminalServer;

/*
 * Example class implementing IDL interface StateChange
 */
class StateChangeSVC_impl
 : public virtual POA_StateChange,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~StateChangeSVC_impl();

 public:
   // standard constructor
   StateChangeSVC_impl();
   virtual ~StateChangeSVC_impl();

   // attributes and operations
   CORBA::Long SetState(CORBA::Long src_id, CORBA::Long state, const char* state_str);
   void			SetParent(TerminalServer *p)	{ pParent = p; }

private:
   TerminalServer	*pParent;

};


/*
 * Example class implementing IDL interface StateChangeFromPA10
 */
class StateChangeFromPA10SVC_impl
 : public virtual POA_StateChangeFromPA10,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~StateChangeFromPA10SVC_impl();

 public:
   // standard constructor
   StateChangeFromPA10SVC_impl();
   virtual ~StateChangeFromPA10SVC_impl();

   // attributes and operations
   CORBA::Long SetState(CORBA::Long state, const char* state_str);
   void			SetParent(TerminalServer *p)	{ pParent = p; }

private:
   TerminalServer	*pParent;
};

/*
 * Example class implementing IDL interface StateChangeFromRH
 */
class StateChangeFromRHSVC_impl
 : public virtual POA_StateChangeFromRH,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~StateChangeFromRHSVC_impl();

 public:
   // standard constructor
   StateChangeFromRHSVC_impl();
   virtual ~StateChangeFromRHSVC_impl();

   // attributes and operations
   CORBA::Long SetState(CORBA::Long state, const char* state_str);
   void			SetParent(TerminalServer *p)	{ pParent = p; }

private:
   TerminalServer	*pParent;
};

/*
 * Example class implementing IDL interface StateChangeFromMeeting
 */
#if 0
class StateChangeFromMeetingSVC_impl
 : public virtual POA_StateChangeFromMeeting,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~StateChangeFromMeetingSVC_impl();

 public:
   // standard constructor
   StateChangeFromMeetingSVC_impl();
   virtual ~StateChangeFromMeetingSVC_impl();

   // attributes and operations
   CORBA::Long SetState(CORBA::Long state, const char* state_str);
   void			SetParent(TerminalServer *p)	{ pParent = p; }

private:
   TerminalServer	*pParent;
};

/*
 * Example class implementing IDL interface StateChangeFromReceptionist
 */
class StateChangeFromReceptionistSVC_impl
 : public virtual POA_StateChangeFromReceptionist,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~StateChangeFromReceptionistSVC_impl();

 public:
   // standard constructor
   StateChangeFromReceptionistSVC_impl();
   virtual ~StateChangeFromReceptionistSVC_impl();

   // attributes and operations
   CORBA::Long SetState(CORBA::Long state, const char* state_str);
   void			SetParent(TerminalServer *p)	{ pParent = p; }

private:
   TerminalServer	*pParent;
};

/*
 * Example class implementing IDL interface ServiceChange
 */

class ServiceChangeSVC_impl
 : public virtual POA_ServiceChange,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~ServiceChangeSVC_impl();

 public:
   // standard constructor
   ServiceChangeSVC_impl();
   virtual ~ServiceChangeSVC_impl();

   // attributes and operations
   CORBA::Long SetService(CORBA::Long src, CORBA::Long service);
   void			SetParent(TerminalServer *p)	{ pParent = p; }

private:
   TerminalServer	*pParent;
};
#endif


#endif // CONTROLTERMINALSVC_IMPL_H


