// -*- C++ -*-
/*!
 * @file  TerminalServer.h
 * @brief ModuleDescription
 * @date  $Date$
 *
 * $Id$
 */

#ifndef TERMINALSERVER_H
#define TERMINALSERVER_H

#ifndef __RS001
#define	__RS001_1
#else
#undef		__RS001_1
#endif

#define SCENARIO_FILE	1

// 標準インクルード
#include <time.h>

#include <vector>

#include <expat.h>

// RTMインクルード
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

#include "QueryInfo.h"
#include "UserMemberData.h"

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "ControlTerminalSVC_impl.h"
#include "PA10CtrlStub.h"
#include "RefHardCtrlProvStub.h"

#define _ALIVE_RTC

#ifdef _ALIVE_RTC
#include "SystemStatus.h"
#endif

#ifdef SCENARIO_FILE

#define SCEANRIO_CMD_MAP	std::map< std::string, int>
#define SCEANRIO_CMD_VAL	SCEANRIO_CMD_MAP::value_type
#define SCEANRIO_CMD_IT	SCEANRIO_CMD_MAP::iterator

#define SCEANRIO_DYNA_PARAM_MAP	std::map< std::string, std::string >
#define SCEANRIO_DYNA_PARAM_VAL	SCEANRIO_DYNA_PARAM_MAP::value_type
#define SCEANRIO_DYNA_PARAM_IT		SCEANRIO_DYNA_PARAM_MAP::iterator

#endif
// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

#if 0
class MessageData{
	std::string	msgText;
	std::string	msgDate;
	std::string	msgAddress;
};
#endif

class TerminalStatus
{
public:
	std::string	ip_addr;
	std::string	attribute;
	std::string	status;
	std::string	camera_ip;
	std::string	mic;
};

#ifdef SCENARIO_FILE
class ScenarioInfo
{
public:
	ScenarioInfo() : commandName(""), commandID(0)	{}
public:
	std::string					commandName;
	int							commandID;
	std::vector<std::string>	param;
//	std::map<long, std::string >	onError;
};
#endif

#define TERMINAL_INFO_MAP	std::map<std::string, TerminalStatus>
#define TERMINAL_INFO_VAL	TERMINAL_INFO_MAP::value_type
#define TERMINAL_INFO_IT	TERMINAL_INFO_MAP::iterator

#define VECTOR_ITERATOR	std::vector< std::string >::iterator


#define TICKET_MAP	std::map< std::string, VisitorTicket>
#define TICKET_VAL	TICKET_MAP::value_type
#define TICKET_IT	TICKET_MAP::iterator

/*!
 * @class TerminalServer
 * @brief ModuleDescription
 *
 */
class TerminalServer
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  TerminalServer(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~TerminalServer();

  // <rtc-template block="public_attribute">

  // </rtc-template>

  // <rtc-template block="public_operation">

  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry()
   *
   * @return RTC::ReturnCode_t
   *
   *
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   *
   *
   */
  virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   *
   *
   */
   virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   *
   *
   */
   virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   *
   *
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   *
   *
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   *
   *
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   *
   *
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   *
   *
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   *
   *
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);

  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   *
   *
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   *
   *
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">

  // </rtc-template>

  // <rtc-template block="protected_operation">

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedString m_rtc_cgi_request;
  InPort<TimedString> m_rtc_cgi_requestIn;

  TimedString m_recv_gtalk_msg;
  InPort<TimedString> m_recv_gtalk_msgIn;
  TimedString m_recv_gtalk_status;
  InPort<TimedString> m_recv_gtalk_statusIn;

#ifdef __RS001
  TimedString m_recv_juliusInside_stat;
  InPort<TimedString> m_recv_juliusInside_statIn;
  TimedString m_recv_juliusInside;
  InPort<TimedString> m_recv_juliusInsideIn;

  TimedString m_recv_juliusOutside_stat;
  InPort<TimedString> m_recv_juliusOutside_statIn;
  TimedString m_recv_juliusOutside;
  InPort<TimedString> m_recv_juliusOutsideIn;

  TimedString m_recv_juliusOrg_stat;
  InPort<TimedString> m_recv_juliusOrg_statIn;
  TimedString m_recv_juliusOrg;
  InPort<TimedString> m_recv_juliusOrgIn;

  TimedString m_recv_juliusNum_stat;
  InPort<TimedString> m_recv_juliusNum_statIn;
  TimedString m_recv_juliusNum;
  InPort<TimedString> m_recv_juliusNumIn;
#endif

#ifdef __RS001_1
  // 音声認識モジュールが複数辞書対応となったので、本モジュールも対応を行ってみる
  TimedString m_recv_julius_stat;
  InPort<TimedString> m_recv_julius_statIn;
  TimedString m_recv_julius;
  InPort<TimedString> m_recv_juliusIn;
#endif

  TimedOctetSeq m_send_juliusData;
  InPort<TimedOctetSeq> m_send_juliusDataIn;

  TimedString m_jtalk_status;
  InPort<TimedString> m_jtalk_statusIn;

  TimedString m_MonitorData;
  InPort<TimedString> m_MonitorDataIn;

  //  TimedOctetSeq m_send_juliusData;	// 音声データ出力ポート用データエリアは、音声データ入力ポートと共有している。
  OutPort<TimedOctetSeq> m_send_juliusDataOut;

  // </rtc-template>
  std::string	m_HttpServerIP;
  float			m_JuliusScore;
  float			m_CommandScore;
  int			m_AnnounceWait;
//  std::string	m_CandidateIndex;
  std::string	m_RtcCentMemberFile;


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedString m_cycle_data;
  OutPort<TimedString> m_cycle_dataOut;

  TimedString m_send_gtalk_msg;
  OutPort<TimedString> m_send_gtalk_msgOut;

  TimedString m_send_voice;
  OutPort<TimedString> m_send_voiceOut;

#ifdef __RS001_1
  TimedString m_julius_grammarData;
  OutPort<TimedString> m_julius_grammar_typeOut;
#endif
  // </rtc-template>

  // CORBA Port declaration
  RTC::CorbaPort m_PA10CtrlPort;
  RTC::CorbaPort m_RHCtrlPort;

  RTC::CorbaPort m_StateChangePA10Port;
  RTC::CorbaPort m_StateChangeRHPort;

  RTC::CorbaConsumer<PA10SystemControl> m_pa10if;
//  RTC::CorbaConsumer<PA10Ctrl> m_pa10if;
  RTC::CorbaConsumer<RefHardCtrl> m_rh_if;

	StateChangeFromPA10SVC_impl			m_StateChangePA10;
	StateChangeFromRHSVC_impl			m_StateChangeRH;
// <rtc-template block="corbaport_declare">

  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">

  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">

  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">

  // </rtc-template>

  // <rtc-template block="private_operation">

  // </rtc-template>
#ifdef SCENARIO_FILE
	void readScenarioFile(std::string fileName, SCEANRIO_DYNA_PARAM_MAP paramList);
#endif
	void createScenario(long num);
	void CheckTicket();
	void VisitorProc(QueryInfo *pQueryInfo);
	void CGI_Proc();
	void GtalkStatusChange();
	void InitializeFlags();
	void InitializeServiceFlags();
	void SendErrorInfoMsg();
	std::vector< std::string > CheckSystemStatus(bool bGetLatest = true);
public:
	long SetTerminalState(long src, long state, std::string state_str);
	long SetServiceState(long src, long state)	{/* long old = ServiceState; ServiceState = state; return old; */ return 0;}
	bool AddCandidate(const std::string name, const float score);
	bool SetMemberStatus(std::string member_addr, bool available);
	void runScenario();

	bool	m_SystemEnd;
	int		m_ScenarioStatus;

private:
	std::string						m_JuliusInsideLastState;
	std::string						m_JuliusInsideLastMsg;
	std::string						m_JuliusOutsideLastState;
	std::string						m_JuliusOutsideLastMsg;
	std::string						m_JuliusOrgLastState;
	std::string						m_JuliusOrgLastMsg;
	std::string						m_JuliusNumLastState;
	std::string						m_JuliusNumLastMsg;
	int			m_Answer;
	bool		m_Visitor;
//	std::vector< VisitorTicket >	m_TicketList;
	TICKET_MAP						m_TicketList;
	MemberInfo						m_MemberInfo;
	XML_Parser						m_parser;
	unsigned int					m_TickerNum;
	TERMINAL_INFO_MAP				m_TerminalList;
	std::vector< std::string >		m_CandidateList;
	int								m_CandidateStage;
	int		m_SystemState;

	pthread_t	m_thread_id;
	long	m_Incoming;				// RH到着シグナル
	long	m_Incoming2;				// RH到着シグナル
	long	m_PA10Waiting;			// PA10ドリンク積載終了
	int		m_DrinkNum;
	bool	m_UserOperation;
	bool	m_UserOperationSave;

	long	m_SystemStatus;
	long	m_Status;
	long	m_pa10Status;
	long	m_rhStatus;
	long	m_FrontDeskStatus;
	long	m_MeetingTermStatus;
	std::vector< long >	m_OldStatusArray;
/*
	long	m_StatusOld;
	long	m_pa10StatusOld;
	long	m_rhStatusOld;
	long	m_FrontDeskStatusOld;
	long	m_MeetingTermStatusOld;
*/
	int		m_stateCheckFreq;

#ifdef SCENARIO_FILE
	std::list<ScenarioInfo>		m_scenarioList;
#else
	std::list<long>		m_scenarioList;
#endif

	time_t	m_Announce;			// 音声再生中フラグ
	std::string		JuliusResult;
	std::string		JTalkStatus;
	std::string		JuliusStatus;

	ScenarioInfo	currentScenario;
	std::string		m_curScenarioName;
	long			m_SystemError;
	long			m_SystemErrorOld;
//	long			m_PA10Error;
//	long			m_RHError;
	int				m_retryCnt;

//	SCEANRIO_CMD_MAP	ScenarioCommand;
#ifdef _ALIVE_RTC
	SystemDataMgr		systemStateData;
#endif
};

extern "C"
{
  DLL_EXPORT void TerminalServerInit(RTC::Manager* manager);

#ifdef SCENARIO_FILE
	void createScenarioCommandList();
#endif
};

#endif // TERMINALSERVER_H
